@@ -434,7 +434,12 @@ class cOrthancServer:
 		#  identifiers). Here is a sample command-line:
 		#  curl -X POST http://localhost:8042/tools/create-media -d '["8c4663df-c3e66066-9e20a8fc-dd14d1e5-251d3d84","2cd4848d-02f0005f-812ffef6-a210bbcf-3f01a00a","6eeded74-75005003-c3ae9738-d4a06a4f-6beedeb8","8a622020-c058291c-7693b63f-bc67aa2e-0a02e69c"]' -v > /tmp/a.zip
 		#  (this will not create duplicates but will also not check for single-patient-ness)
-		f.write(self.__run_POST(url = url, data = study_ids))
+		try:
+			f.write(self.__run_POST(url = url, data = study_ids))
+		except TypeError:
+			f.close()
+			_log.exception('cannot retrieve multiple studies as one archive with DICOMDIR, probably not supported by this Orthanc version')
+			return False
 		f.close()
 		if create_zip:
 			return filename
