@@ -14,7 +14,7 @@ import sys
 import logging
 import shutil
 import os
-import codecs
+import io
 
 
 if __name__ == '__main__':
@@ -299,7 +299,13 @@ _html_list_item = u"""	<li><a href="documents/%s">%s</a></li>
 _html_end = u"""
 </ul>
 
-%s, GNUmed version %s
+<p>
+%(date)s<br>
+%(branch)s @ %(praxis)s
+%(adr)s
+</p>
+
+(<a href="http://www.gnumed.de">GNUmed</a> version %(gm_ver)s)
 
 </body>
 </html>
@@ -388,7 +394,7 @@ class cExportArea(object):
 	def add_file(self, filename=None, hint=None):
 		try:
 			open(filename).close()
-		except StandardError:
+		except Exception:
 			_log.exception('cannot open file <%s>', filename)
 			return None
 
@@ -459,11 +465,12 @@ class cExportArea(object):
 
 		r = rows[0]
 		return cExportItem(row = {'data': r, 'idx': idx, 'pk_field': 'pk_export_item'})
+
 	#--------------------------------------------------------
-	def export_with_meta_data(self, base_dir=None, items=None):
+	def export(self, base_dir=None, items=None, with_metadata=True, expand_compressed=False):
 
 		if items is None:
-			items_found = self.items
+			items = self.items
 
 		if len(items) == 0:
 			return None
@@ -471,46 +478,41 @@ class cExportArea(object):
 		from Gnumed.business.gmPerson import cPatient
 		pat = cPatient(aPK_obj = self.__pk_identity)
 		if base_dir is None:
-			base_dir = gmTools.get_unique_filename(prefix = u'export-%s-' % pat.dirname, suffix = '.dir')
-
+			base_dir = gmTools.mk_sandbox_dir(prefix = u'exp-%s-' % pat.dirname)
 		_log.debug('base dir: %s', base_dir)
 
 		doc_dir = os.path.join(base_dir, r'documents')
 		gmTools.mkdir(doc_dir)
 
-		mugshot = pat.document_folder.latest_mugshot
-		if mugshot is None:
-			mugshot_url = u'documents/no-such-file.png'
-			mugshot_alt = _('no patient photograph available')
-			mugshot_title = u''
-		else:
-			mugshot_url = mugshot.export_to_file(directory = doc_dir)
-			mugshot_alt =_('patient photograph from %s') % gmDateTime.pydt_strftime(mugshot['date_generated'], '%B %Y')
-			mugshot_title = gmDateTime.pydt_strftime(mugshot['date_generated'], '%B %Y')
-
-		# index.html
-		idx_fname = os.path.join(base_dir, u'index.html')
-		idx_file = codecs.open(idx_fname, u'wb', u'utf8')
-
-		# header
-		browse_dicomdir = u''
-		existing_files = os.listdir(base_dir)
-		if u'DICOMDIR' in existing_files:
-			browse_dicomdir = u'	<li><a href="./DICOMDIR">browse DICOMDIR</a></li>'
-		idx_file.write(_html_start % {
+		_html_start_data = {
 			u'html_title_header': _('Patient data for'),
-			u'html_title_patient': gmTools.html_escape_string(pat['description_gender'] + u', ' + _(u'born') + u' ' + pat.get_formatted_dob('%Y %B %d')),
+			u'html_title_patient': gmTools.html_escape_string(pat.get_description_gender(with_nickname = False) + u', ' + _(u'born') + u' ' + pat.get_formatted_dob('%Y %B %d')),
 			u'title': _('Patient data export'),
-			u'pat_name': gmTools.html_escape_string(pat['description_gender']),
+			u'pat_name': gmTools.html_escape_string(pat.get_description_gender(with_nickname = False)),
 			u'pat_dob': gmTools.html_escape_string(_(u'born') + u' ' + pat.get_formatted_dob('%Y %B %d')),
-			u'mugshot_url': mugshot_url,
-			u'mugshot_alt': mugshot_alt,
-			u'mugshot_title': mugshot_title,
+			u'mugshot_url': u'documents/no-such-file.png',
+			u'mugshot_alt': _('no patient photograph available'),
+			u'mugshot_title': u'',
 			u'docs_title': _(u'Documents'),
 			u'browse_root': _(u'browse storage medium'),
 			u'browse_docs': _(u'browse documents area'),
-			u'browse_dicomdir': browse_dicomdir
-		})
+			u'browse_dicomdir': u''
+		}
+
+		mugshot = pat.document_folder.latest_mugshot
+		if mugshot is not None:
+			_html_start_data['mugshot_url'] = mugshot.export_to_file(directory = doc_dir)
+			_html_start_data['mugshot_alt'] =_('patient photograph from %s') % gmDateTime.pydt_strftime(mugshot['date_generated'], '%B %Y')
+			_html_start_data['mugshot_title'] = gmDateTime.pydt_strftime(mugshot['date_generated'], '%B %Y')
+
+		# index.html
+		idx_fname = os.path.join(base_dir, u'index.html')
+		idx_file = io.open(idx_fname, mode = u'wt', encoding = u'utf8')
+		# header
+		existing_files = os.listdir(base_dir)
+		if u'DICOMDIR' in existing_files:
+			_html_start_data[u'browse_dicomdir'] = u'	<li><a href="./DICOMDIR">browse DICOMDIR</a></li>'
+		idx_file.write(_html_start % _html_start_data)
 		# middle
 		for item in items:
 			item_path = item.export_to_file(directory = doc_dir)
@@ -521,30 +523,51 @@ class cExportArea(object):
 			))
 		# footer
 		_cfg = gmCfg2.gmCfgData()
-		idx_file.write(_html_end % (
-			gmTools.html_escape_string(gmDateTime.pydt_strftime(format = '%Y %B %d', encoding = u'utf8')),
-			gmTools.html_escape_string(_cfg.get(option = u'client_version'))
-		))
+		from Gnumed.business.gmPraxis import gmCurrentPraxisBranch
+		prax = gmCurrentPraxisBranch()
+		lines = []
+		adr = prax.branch.org_unit.address
+		if adr is not None:
+			lines.extend(adr.format())
+		for comm in prax.branch.org_unit.comm_channels:
+			if comm['is_confidential'] is True:
+				continue
+			lines.append(u'%s: %s' % (
+				comm['l10n_comm_type'],
+				comm['url']
+			))
+		adr = u''
+		if len(lines) > 0:
+			adr = gmTools.html_escape_string(u'\n'.join(lines), replace_eol = True, keep_visual_eol = True)
+		_html_end_data = {
+			'branch': gmTools.html_escape_string(prax['branch']),
+			'praxis': gmTools.html_escape_string(prax['praxis']),
+			'date' : gmTools.html_escape_string(gmDateTime.pydt_strftime(gmDateTime.pydt_now_here(), format = '%Y %B %d', encoding = u'utf8')),
+			'gm_ver': gmTools.html_escape_string(_cfg.get(option = u'client_version')),
+			#'gm_ver': 'git HEAD',				# for testing
+			'adr': adr
+		}
+		idx_file.write(_html_end % _html_end_data)
 		idx_file.close()
 
 		# autorun.inf
 		autorun_fname = os.path.join(base_dir, u'autorun.inf')
-		autorun_file = codecs.open(autorun_fname, u'wb', u'utf8')
+		autorun_file = io.open(autorun_fname, mode = u'wt', encoding = u'utf8')
 		autorun_file.write(_autorun_inf % (
-			(pat['description_gender'] + u', ' + _(u'born') + u' ' + pat.get_formatted_dob('%Y %B %d')).strip(),
+			(pat.get_description_gender(with_nickname = False) + u', ' + _(u'born') + u' ' + pat.get_formatted_dob('%Y %B %d')).strip(),
 			_('Browse patient data')
 		))
 		autorun_file.close()
 
 		# cd.inf
 		cd_inf_fname = os.path.join(base_dir, u'cd.inf')
-		cd_inf_file = codecs.open(cd_inf_fname, u'wb', u'utf8')
+		cd_inf_file = io.open(cd_inf_fname, mode = u'wt', encoding = u'utf8')
 		cd_inf_file.write(_cd_inf % (
 			pat['lastnames'],
 			pat['firstnames'],
 			gmTools.coalesce(pat['gender'], u'?'),
 			pat.get_formatted_dob('%Y-%m-%d'),
-			gmDateTime.pydt_strftime(format = '%Y-%m-%d', encoding = u'utf8'),
+			gmDateTime.pydt_strftime(gmDateTime.pydt_now_here(), format = '%Y-%m-%d', encoding = u'utf8'),
 			pat.ID,
 			_cfg.get(option = u'client_version'),
 			u' / '.join([ u'%s = %s (%s)' % (g['tag'], g['label'], g['l10n_label']) for g in pat.gender_list ])
@@ -553,9 +576,9 @@ class cExportArea(object):
 
 		# README
 		readme_fname = os.path.join(base_dir, u'README')
-		readme_file = codecs.open(readme_fname, u'wb', u'utf8')
+		readme_file = io.open(readme_fname, mode = u'wt', encoding = u'utf8')
 		readme_file.write(_README % (
-			pat['description_gender'] + u', ' + _(u'born') + u' ' + pat.get_formatted_dob('%Y %B %d')
+			pat.get_description_gender(with_nickname = False) + u', ' + _(u'born') + u' ' + pat.get_formatted_dob('%Y %B %d')
 		))
 		readme_file.close()
 
@@ -564,6 +587,9 @@ class cExportArea(object):
 		pat.export_as_xml_linuxmednews(filename = os.path.join(base_dir, u'patient.xml'))
 		pat.export_as_vcard(filename = os.path.join(base_dir, u'patient.vcf'))
 
+		# praxis VCF
+		shutil.move(prax.vcf, os.path.join(base_dir, u'praxis.vcf'))
+
 		return base_dir
 	#--------------------------------------------------------
 	# properties
@@ -588,10 +614,11 @@ if __name__ == '__main__':
 		sys.exit()
 
 	from Gnumed.pycommon import gmI18N
-
 	gmI18N.activate_locale()
 	gmI18N.install_domain()
 
+	from Gnumed.business import gmPraxis
+
 	#---------------------------------------
 	def test_export_items():
 #		items = get_export_items()
@@ -609,7 +636,12 @@ if __name__ == '__main__':
 	def test_export_area():
 		exp = cExportArea(12)
 		#print exp.export_with_meta_data()
-		print exp.items
+		#print exp.items
+		exp.add_file(sys.argv[2])
+		prax = gmPraxis.gmCurrentPraxisBranch(branch = gmPraxis.cPraxisBranch(1))
+		print prax
+		print prax.branch
+		print exp.export(with_metadata = True)
 	#---------------------------------------
 	#test_export_items()
 	test_export_area()
