@@ -1,35 +1,37 @@
-"""GNUmed simple ASCII EMR export tool.
+__doc__ = """GNUmed simple ASCII EMR export tool.
 
 TODO:
-- GUI mode:
-  - post-0.1 !
-  - allow user to select patient
-  - allow user to pick episodes/encounters/etc from list
 - output modes:
   - HTML - post-0.1 !
 """
 #============================================================
-__author__ = "Carlos Moro"
-__license__ = 'GPL'
+__author__ = "Carlos Moro, Karsten Hilbert"
+__license__ = 'GPL v2 or later'
 
-import os.path, sys, types, time, codecs, datetime as pyDT, logging, shutil
-
-
-import mx.DateTime.Parser as mxParser
-import mx.DateTime as mxDT
+import os.path
+import sys
+import types
+import time
+import io
+import logging
+import shutil
 
 
 if __name__ == '__main__':
 	sys.path.insert(0, '../../')
-
-from Gnumed.pycommon import gmI18N
-
-if __name__ == '__main__':
+	from Gnumed.pycommon import gmI18N
 	gmI18N.activate_locale()
 	gmI18N.install_domain()
 
-from Gnumed.pycommon import gmExceptions, gmNull, gmPG2, gmTools, gmDateTime
-from Gnumed.business import gmClinicalRecord, gmPerson, gmAllergy, gmDemographicRecord, gmClinNarrative, gmPersonSearch
+from Gnumed.pycommon import gmExceptions
+from Gnumed.pycommon import gmPG2
+from Gnumed.pycommon import gmTools
+from Gnumed.pycommon import gmDateTime
+
+from Gnumed.business import gmClinicalRecord
+from Gnumed.business import gmAllergy
+from Gnumed.business import gmDemographicRecord
+from Gnumed.business import gmSoapDefs
 
 
 _log = logging.getLogger('gm.export')
@@ -244,16 +246,16 @@ class cEmrExport:
                 if len(vaccinations[indication]) > vacc_regimes[cont]['shots']: # boosters given
                     all_vreg_boosters[cont] = vaccinations[indication][len(vaccinations[indication])-1]['date'].strftime('%Y-%m-%d') # show last given booster date
                     scheduled_booster = vaccinations4regimes[indication][len(vaccinations4regimes[indication])-1]
-                    booster_date = vaccinations[indication][len(vaccinations[indication])-1]['date'] + scheduled_booster['min_interval']                                        
-                    if booster_date < mxDT.today():
+                    booster_date = vaccinations[indication][len(vaccinations[indication])-1]['date'] + scheduled_booster['min_interval']
+                    if booster_date < gmDateTime.pydt_now_here():
                         all_next_boosters[cont] = '<(' + booster_date.strftime('%Y-%m-%d') + ')>' # next booster is due
                     else:
                         all_next_boosters[cont] = booster_date.strftime('%Y-%m-%d')
                 elif len(vaccinations[indication]) == vacc_regimes[cont]['shots']: # just finished vaccinations, begin boosters
                     all_vreg_boosters[cont] = column_widths[cont+1] * ' '
                     scheduled_booster = vaccinations4regimes[indication][len(vaccinations4regimes[indication])-1]
-                    booster_date = vaccinations[indication][len(vaccinations[indication])-1]['date'] + scheduled_booster['min_interval']                    
-                    if booster_date < mxDT.today():
+                    booster_date = vaccinations[indication][len(vaccinations[indication])-1]['date'] + scheduled_booster['min_interval']
+                    if booster_date < gmDateTime.pydt_now_here():
                         all_next_boosters[cont] = '<(' + booster_date.strftime('%Y-%m-%d') + ')>' # next booster is due
                     else:
                         all_next_boosters[cont] = booster_date.strftime('%Y-%m-%d')
@@ -761,10 +763,10 @@ class cEmrExport:
         if len(unlinked_episodes) > 0:
             h_issues.insert(0, {'description':_('Unattributed episodes'), 'pk_health_issue':None})        
         for a_health_issue in h_issues:
-            self.__target.write('\n' + 3*' ' + 'Health Issue: ' + a_health_issue['description'] + '\n')
+            self.__target.write(u'\n' + 3*' ' + 'Health Issue: ' + a_health_issue['description'] + '\n')
             episodes = emr.get_episodes(id_list=self.__constraints['episodes'], issues = [a_health_issue['pk_health_issue']])
             for an_episode in episodes:
-               self.__target.write('\n' + 6*' ' + 'Episode: ' + an_episode['description'] + '\n')
+               self.__target.write(u'\n' + 6*' ' + 'Episode: ' + an_episode['description'] + '\n')
                if a_health_issue['pk_health_issue'] is None:
                   issues = None
                else:
@@ -803,14 +805,14 @@ class cEmrExport:
                 'Please check the log file for details.'
             ))
             return None
-        self.__target.write('\nOverview\n')
-        self.__target.write('--------\n')
-        self.__target.write("1) Allergy status (for details, see below):\n\n")
+        self.__target.write(u'\nOverview\n')
+        self.__target.write(u'--------\n')
+        self.__target.write(u"1) Allergy status (for details, see below):\n\n")
         for allergy in       emr.get_allergies():
-            self.__target.write("    " + allergy['descriptor'] + "\n\n")
-        self.__target.write("2) Vaccination status (* indicates booster):\n")
+            self.__target.write(u"    " + allergy['descriptor'] + "\n\n")
+        self.__target.write(u"2) Vaccination status (* indicates booster):\n")
 #        self.get_vacc_table()
-        self.__target.write("\n3) Historical:\n\n")
+        self.__target.write(u"\n3) Historical:\n\n")
         self.dump_historical_tree()
 
         try:
@@ -825,23 +827,23 @@ class cEmrExport:
         """
         doc_folder = self.__patient.get_document_folder()
 
-        self.__target.write('\n4) Medical documents: (date) reference - type "comment"\n')
-        self.__target.write('                          object - comment')
+        self.__target.write(u'\n4) Medical documents: (date) reference - type "comment"\n')
+        self.__target.write(u'                          object - comment')
 
         docs = doc_folder.get_documents()
         for doc in docs:
-            self.__target.write('\n\n    (%s) %s - %s "%s"' % (
+            self.__target.write(u'\n\n    (%s) %s - %s "%s"' % (
                 doc['clin_when'].strftime('%Y-%m-%d'),
                 doc['ext_ref'],
                 doc['l10n_type'],
                 doc['comment'])
             )
             for part in doc.parts:
-                self.__target.write('\n         %s - %s' % (
+                self.__target.write(u'\n         %s - %s' % (
                     part['seq_idx'],
                     part['obj_comment'])
                 )
-        self.__target.write('\n\n')
+        self.__target.write(u'\n\n')
     #--------------------------------------------------------     
     def dump_demographic_record(self, all = False):
         """
@@ -855,20 +857,20 @@ class cEmrExport:
             ))
             return None
 
-        self.__target.write('\n\n\nDemographics')
-        self.__target.write('\n------------\n')
-        self.__target.write('    Id: %s \n' % self.__patient['pk_identity'])
+        self.__target.write(u'\n\n\nDemographics')
+        self.__target.write(u'\n------------\n')
+        self.__target.write(u'    Id: %s \n' % self.__patient['pk_identity'])
         cont = 0
         for name in self.__patient.get_names():
             if cont == 0:
-                self.__target.write('    Name (Active): %s, %s\n' % (name['firstnames'], name['lastnames']) )
+                self.__target.write(u'    Name (Active): %s, %s\n' % (name['firstnames'], name['lastnames']) )
             else:
-                self.__target.write('    Name %s: %s, %s\n' % (cont, name['firstnames'], name['lastnames']))
+                self.__target.write(u'    Name %s: %s, %s\n' % (cont, name['firstnames'], name['lastnames']))
             cont += 1
-        self.__target.write('    Gender: %s\n' % self.__patient['gender'])
-        self.__target.write('    Title: %s\n' % self.__patient['title'])
-        self.__target.write('    Dob: %s\n' % self.__patient.get_formatted_dob(format = '%Y-%m-%d'))
-        self.__target.write('    Medical age: %s\n' % self.__patient.get_medical_age())
+        self.__target.write(u'    Gender: %s\n' % self.__patient['gender'])
+        self.__target.write(u'    Title: %s\n' % self.__patient['title'])
+        self.__target.write(u'    Dob: %s\n' % self.__patient.get_formatted_dob(format = '%Y-%m-%d'))
+        self.__target.write(u'    Medical age: %s\n' % self.__patient.get_medical_age())
     #--------------------------------------------------------
     def dump_constraints(self):
         """
@@ -877,27 +879,27 @@ class cEmrExport:
         self.__first_constraint = True
         if not self.__constraints['since'] is None:
             self.dump_constraints_header()
-            self.__target.write('\nSince: %s' % self.__constraints['since'].strftime('%Y-%m-%d'))
+            self.__target.write(u'\nSince: %s' % self.__constraints['since'].strftime('%Y-%m-%d'))
 
         if not self.__constraints['until'] is None:
             self.dump_constraints_header()
-            self.__target.write('\nUntil: %s' % self.__constraints['until'].strftime('%Y-%m-%d'))
+            self.__target.write(u'\nUntil: %s' % self.__constraints['until'].strftime('%Y-%m-%d'))
 
         if not self.__constraints['encounters'] is None:
             self.dump_constraints_header()
-            self.__target.write('\nEncounters: ')
+            self.__target.write(u'\nEncounters: ')
             for enc in self.__constraints['encounters']:
                 self.__target.write(str(enc) + ' ')
 
         if not self.__constraints['episodes'] is None:
             self.dump_constraints_header()
-            self.__target.write('\nEpisodes: ')
+            self.__target.write(u'\nEpisodes: ')
             for epi in self.__constraints['episodes']:
                 self.__target.write(str(epi) + ' ')
 
         if not self.__constraints['issues'] is None:
             self.dump_constraints_header()
-            self.__target.write('\nIssues: ')
+            self.__target.write(u'\nIssues: ')
             for iss in self.__constraints['issues']:
                 self.__target.write(str(iss) + ' ')
     #--------------------------------------------------------
@@ -906,41 +908,39 @@ class cEmrExport:
             Dumps constraints header
         """
         if self.__first_constraint == True:
-            self.__target.write('\nClinical items dump constraints\n')
-            self.__target.write('-'*(len(head_txt)-2))
+            self.__target.write(u'\nClinical items dump constraints\n')
+            self.__target.write(u'-'*(len(head_txt)-2))
             self.__first_constraint = False
+
 #============================================================
 class cEMRJournalExporter:
 	"""Exports patient EMR into a simple chronological journal.
-
-	Note that this export will emit u'' strings only.
 	"""
 	def __init__(self):
-		self.__part_len = 72
+		self.__narrative_wrap_len = 72
+
 	#--------------------------------------------------------
 	# external API
 	#--------------------------------------------------------
 	def export_to_file_by_mod_time(self, filename=None, patient=None):
 		if patient is None:
-			patient = gmPerson.gmCurrentPatient()
-			if not patient.connected:
-				raise ValueError('[%s].export_to_file(): no active patient' % self.__class__.__name__)
+			raise ValueError('[%s].export_to_file_by_mod_time(): no patient' % self.__class__.__name__)
 
 		if filename is None:
 			filename = gmTools.get_unique_filename(prefix = 'gm-emr_by_mod_time-', suffix = '.txt')
 
-		f = codecs.open(filename = filename, mode = 'w+b', encoding = 'utf8', errors = 'replace')
+		f = io.open(filename, mode = 'w+t', encoding = 'utf8', errors = 'replace')
 
-		self.__part_len = 80
+		self.__narrative_wrap_len = 80
 
 		# write header
 		txt = _('EMR Journal sorted by last modification time\n')
 		f.write(txt)
 		f.write(u'=' * (len(txt)-1))
-		f.write('\n')
+		f.write(u'\n')
 		f.write(_('Patient: %s (%s), No: %s\n') % (patient['description'], patient['gender'], patient['pk_identity']))
 		f.write(_('Born   : %s, age: %s\n\n') % (
-			patient.get_formatted_dob(format = '%Y %b %d', encoding = gmI18N.get_encoding()),
+			patient.get_formatted_dob(format = '%Y %b %d', encoding = 'utf8'),
 			patient.get_medical_age()
 		))
 
@@ -955,86 +955,112 @@ class cEMRJournalExporter:
 		"""
 		rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': {'pat': patient['pk_identity']}}])
 
-		f.write ((u'-' * 100) + u'\n')
-		f.write (u'%16.16s | %9.9s | %1.1s | %s \n' % (_('Last modified'), _('By'), u' ', _('Entry')))
-		f.write ((u'-' * 100) + u'\n')
+		f.write ((gmTools.u_box_horiz_single * 100) + u'\n')
+		f.write (u'%16.16s %s %9.9s %s %1.1s %s %s \n' % (
+			_('Last modified'),
+			gmTools.u_box_vert_light,
+			_('By'),
+			gmTools.u_box_vert_light,
+			u' ',
+			gmTools.u_box_vert_light,
+			_('Entry')
+		))
+		f.write ((gmTools.u_box_horiz_single * 100) + u'\n')
 
 		for r in rows:
-			txt = u'%16.16s | %9.9s | %1.1s | %s \n' % (
+			txt = u'%16.16s %s %9.9s %s %1.1s %s %s \n' % (
 				r['date_modified'],
+				gmTools.u_box_vert_light,
 				r['modified_by'],
-				gmClinNarrative.soap_cat2l10n[r['soap_cat']],
+				gmTools.u_box_vert_light,
+				gmSoapDefs.soap_cat2l10n[r['soap_cat']],
+				gmTools.u_box_vert_light,
 				gmTools.wrap (
 					text = r['narrative'].replace(u'\r', u'') + u' (%s: %s)' % (_('When'), gmDateTime.pydt_strftime(r['clin_when'], '%Y %b %d %H:%M')),
-					width = self.__part_len,
-					subsequent_indent = u'%31.31s%1.1s | ' % (u' ', gmClinNarrative.soap_cat2l10n[r['soap_cat']])
+					width = self.__narrative_wrap_len,
+					subsequent_indent = u'%31.31s%1.1s %s ' % (u' ', gmSoapDefs.soap_cat2l10n[r['soap_cat']], gmTools.u_box_vert_light)
 				)
 			)
 			f.write(txt)
 
-		f.write((u'-' * 100) + u'\n\n')
+		f.write ((gmTools.u_box_horiz_single * 100) + u'\n')
 		f.write(_('Exported: %s\n') % gmDateTime.pydt_strftime(gmDateTime.pydt_now_here(), '%Y %b %d  %H:%M:%S'))
 
 		f.close()
 		return filename
+
 	#--------------------------------------------------------
-	def export_to_file(self, filename=None, patient=None):
+	def export_to_file_by_encounter(self, filename=None, patient=None):
 		"""Export medical record into a file.
 
 		@type filename: None (creates filename by itself) or string
-		@type patient: None (use currently active patient) or <gmPerson.cIdentity> instance
+		@type patient: <cPerson> instance
 		"""
 		if patient is None:
-			patient = gmPerson.gmCurrentPatient()
-			if not patient.connected:
-				raise ValueError('[%s].export_to_file(): no active patient' % self.__class__.__name__)
+			raise ValueError('[%s].export_to_file_by_encounter(): no patient' % self.__class__.__name__)
 
 		if filename is None:
-			filename = u'%s-%s-%s-%s.txt' % (
-				_('emr-journal'),
-				patient['lastnames'].replace(u' ', u'_'),
-				patient['firstnames'].replace(u' ', u'_'),
-				patient.get_formatted_dob(format = '%Y-%m-%d')
-			)
-			path = os.path.expanduser(os.path.join('~', 'gnumed', patient['dirname'], filename))
+			filename = gmTools.get_unique_filename(prefix = 'gm-emr_journal-', suffix = '.txt')
 
-		f = codecs.open(filename = filename, mode = 'w+b', encoding = 'utf8', errors = 'replace')
-		self.export(target = f, patient = patient)
+		f = io.open(filename, mode = 'w+t', encoding = 'utf8', errors = 'replace')
+		self.__export_by_encounter(target = f, patient = patient)
 		f.close()
 		return filename
+
 	#--------------------------------------------------------
 	# internal API
 	#--------------------------------------------------------
-	def export(self, target=None, patient=None):
+	def __export_by_encounter(self, target=None, patient=None):
 		"""
 		Export medical record into a Python object.
 
 		@type target: a python object supporting the write() API
-		@type patient: None (use currently active patient) or <gmPerson.cIdentity> instance
+		@type patient: <cPerson> instance
 		"""
-		if patient is None:
-			patient = gmPerson.gmCurrentPatient()
-			if not patient.connected:
-				raise ValueError('[%s].export(): no active patient' % self.__class__.__name__)
-
-		# write header
-		txt = _('Chronological EMR Journal\n')
+		txt = _(u'Chronological EMR Journal\n')
 		target.write(txt)
 		target.write(u'=' * (len(txt)-1))
-		target.write('\n')
-		target.write(_('Patient: %s (%s), No: %s\n') % (patient['description'], patient['gender'], patient['pk_identity']))
-		target.write(_('Born   : %s, age: %s\n\n') % (
-			patient.get_formatted_dob(format = '%Y %b %d', encoding = gmI18N.get_encoding()),
+		target.write(u'\n')
+		# demographics
+		target.write(_(u'Patient: %s (%s), No: %s\n') % (patient['description'], patient['gender'], patient['pk_identity']))
+		target.write(_(u'Born   : %s, age: %s\n\n') % (
+			patient.get_formatted_dob(format = '%Y %b %d', encoding = 'utf8'),
 			patient.get_medical_age()
 		))
 		for ext_id in patient.external_ids:
 			target.write(u'%s: %s (@%s)\n' % (ext_id['name'], ext_id['value'], ext_id['issuer']))
 		for ch in patient.comm_channels:
 			target.write(u'%s: %s\n' % (ch['l10n_comm_type'], ch['url']))
-		target.write(u'.-%10.10s---%9.9s-------%72.72s\n' % (u'-' * 10, u'-' * 9, u'-' * self.__part_len))
-		target.write(u'| %10.10s | %9.9s |     | %s\n' % (_('Encounter'), _('Doc'), _('Narrative')))
-		target.write(u'|-%10.10s---%9.9s-------%72.72s\n' % (u'-' * 10, u'-' * 9, u'-' * self.__part_len))
-
+		# table header
+		target.write(u'%s%12.12s%s%11.11s%s%s%s%72.72s\n' % (
+			gmTools.u_box_top_left_arc,
+			gmTools.u_box_horiz_single * 12,
+			gmTools.u_box_T_down,
+			gmTools.u_box_horiz_single * 11,
+			gmTools.u_box_T_down,
+			gmTools.u_box_horiz_single * 5,
+			gmTools.u_box_T_down,
+			gmTools.u_box_horiz_single * self.__narrative_wrap_len
+		))
+		target.write(u'%s %10.10s %s %9.9s %s     %s %s\n' % (
+			gmTools.u_box_vert_light,
+			_(u'Encounter'),
+			gmTools.u_box_vert_light,
+			_(u'Doc'),
+			gmTools.u_box_vert_light,
+			gmTools.u_box_vert_light,
+			_(u'Narrative')
+		))
+		target.write(u'%s%12.12s%s%11.11s%s%s%s%72.72s\n' % (
+			gmTools.u_box_T_right,
+			gmTools.u_box_horiz_single * 12,
+			gmTools.u_box_plus,
+			gmTools.u_box_horiz_single * 11,
+			gmTools.u_box_plus,
+			gmTools.u_box_horiz_single * 5,
+			gmTools.u_box_plus,
+			gmTools.u_box_horiz_single * self.__narrative_wrap_len
+		))
 		# get data
 		cmd = u"""
 			SELECT
@@ -1059,7 +1085,7 @@ class cEMRJournalExporter:
 
 			txt = gmTools.wrap (
 				text = row['narrative'].replace(u'\r', u'') + (u' (%s)' % row['date_modified']),
-				width = self.__part_len
+				width = self.__narrative_wrap_len
 			).split('\n')
 
 			# same provider ?
@@ -1086,10 +1112,14 @@ class cEMRJournalExporter:
 				curr_date = u''
 
 			# display first part
-			target.write(u'| %10.10s | %9.9s | %3.3s | %s\n' % (
+			target.write(u'%s %10.10s %s %9.9s %s %3.3s %s %s\n' % (
+				gmTools.u_box_vert_light,
 				curr_date,
+				gmTools.u_box_vert_light,
 				curr_doc,
-				gmClinNarrative.soap_cat2l10n[curr_soap],
+				gmTools.u_box_vert_light,
+				gmSoapDefs.soap_cat2l10n[curr_soap],
+				gmTools.u_box_vert_light,
 				txt[0]
 			))
 
@@ -1097,26 +1127,45 @@ class cEMRJournalExporter:
 			if len(txt) == 1:
 				continue
 
-			template = u'| %10.10s | %9.9s | %3.3s | %s\n'
+			template = u'%s %10.10s %s %9.9s %s %3.3s %s %s\n'
 			for part in txt[1:]:
-				line = template % (u'', u'', u' ', part)
+				line = template % (
+					gmTools.u_box_vert_light,
+					u'',
+					gmTools.u_box_vert_light,
+					u'',
+					gmTools.u_box_vert_light,
+					u' ',
+					gmTools.u_box_vert_light,
+					part
+				)
 				target.write(line)
 
 		# write footer
-		target.write(u'`-%10.10s---%9.9s-------%72.72s\n\n' % (u'-' * 10, u'-' * 9, u'-' * self.__part_len))
-		target.write(_('Exported: %s\n') % gmDateTime.pydt_strftime(pyDT.datetime.now(), '%Y %b %d  %H:%M:%S'))
+		target.write(u'%s%12.12s%s%11.11s%s%5.5s%s%72.72s\n\n' % (
+			gmTools.u_box_bottom_left_arc,
+			gmTools.u_box_horiz_single * 12,
+			gmTools.u_box_T_up,
+			gmTools.u_box_horiz_single * 11,
+			gmTools.u_box_T_up,
+			gmTools.u_box_horiz_single * 5,
+			gmTools.u_box_T_up,
+			gmTools.u_box_horiz_single * self.__narrative_wrap_len
+		))
+
+		target.write(_(u'Exported: %s\n') % gmDateTime.pydt_strftime(gmDateTime.pydt_now_here(), format = '%Y %b %d  %H:%M:%S'))
 
 		return
+
 #============================================================
 class cMedistarSOAPExporter:
 	"""Export SOAP data per encounter into Medistar import format."""
+
 	def __init__(self, patient=None):
 		if patient is None:
-			self.__pat = gmPerson.gmCurrentPatient()
-		else:
-			if not isinstance(patient, gmPerson.cIdentity):
-				raise gmExceptions.ConstructorError, '<patient> argument must be instance of <cIdentity>, but is: %s' % type(patient)
-			self.__pat = patient
+			raise gmExceptions.ConstructorError, '<patient> argument must be instance of <cPerson>, but is: %s' % type(patient)
+		self.__pat = patient
+
 	#--------------------------------------------------------
 	# external API
 	#--------------------------------------------------------
@@ -1134,7 +1183,7 @@ class cMedistarSOAPExporter:
 				self.__pat.get_formatted_dob(format = '%Y-%m-%d')
 			)
 
-		f = codecs.open(filename = filename, mode = 'w+b', encoding = 'cp437', errors='replace')
+		f = io.open(filename, mode = 'w+t', encoding = 'cp437', errors='replace')
 		status = self.__export(target = f, encounter = encounter, soap_cats = soap_cats)
 		f.close()
 
@@ -1169,17 +1218,18 @@ class cMedistarSOAPExporter:
 		cmd = u"select narrative from clin.v_emr_journal where pk_patient=%s and pk_encounter=%s and soap_cat=%s"
 		for soap_cat in soap_cats:
 			rows, idx = gmPG2.run_ro_queries(queries = [{'cmd': cmd, 'args': (self.__pat['pk_identity'], encounter['pk_encounter'], soap_cat)}])
-			target.write('*MD%s*\r\n' % gmClinNarrative.soap_cat2l10n[soap_cat])
+			target.write(u'*MD%s*\r\n' % gmSoapDefs.soap_cat2l10n[soap_cat])
 			for row in rows:
 				text = row[0]
 				if text is None:
 					continue
-				target.write('%s\r\n' % gmTools.wrap (
+				target.write(u'%s\r\n' % gmTools.wrap (
 					text = text,
 					width = 64,
 					eol = u'\r\n'
 				))
 		return True
+
 #============================================================
 # main
 #------------------------------------------------------------
@@ -1206,7 +1256,6 @@ def run():
         if patient is None:
             break
         # FIXME: needed ?
-#        gmPerson.set_active_patient(patient=patient)
         exporter = cEMRJournalExporter()
         exporter.export_to_file(patient=patient)
 #        export_tool.set_patient(patient)
@@ -1224,12 +1273,13 @@ def run():
             patient.cleanup()
         except:
             print "error cleaning up patient"
+
 #============================================================
 # main
 #------------------------------------------------------------
 if __name__ == "__main__":
-	gmI18N.activate_locale()
-	gmI18N.install_domain()
+
+	from Gnumed.business import gmPersonSearch
 
 	#--------------------------------------------------------
 	def export_journal():
@@ -1242,7 +1292,7 @@ if __name__ == "__main__":
 				break
 
 			exporter = cEMRJournalExporter()
-			print "exported into file:", exporter.export_to_file(patient=patient)
+			print "exported into file:", exporter.export_to_file_by_encounter(patient = patient)
 
 			if patient is not None:
 				try:
