@@ -18,6 +18,7 @@ __license__ = 'GPL v2 or later (details at http://www.gnu.org)'
 import time
 import sys
 import os
+import io
 import codecs
 import types
 import logging
@@ -87,6 +88,8 @@ FixedOffsetTimezone = dbapi.tz.FixedOffsetTimezone
 _default_dsn = None
 _default_login = None
 
+default_database = 'gnumed_v21'
+
 postgresql_version_string = None
 postgresql_version = None			# accuracy: major.minor
 
@@ -119,7 +122,8 @@ known_schema_hashes = {
 	#19: '419e5225259c53dd36ad80d82066ff02'	# 19.0 only
 	#19: '9765373098b03fb208332498f34cd4b5' # until 19.11
 	19: '57f009a159f55f77525cc0291e0c8b60', # starting with 19.12
-	20: 'baed1901ed4c2f272b56c8cb2c6d88e8'
+	20: 'baed1901ed4c2f272b56c8cb2c6d88e8',
+	21: 'e6a51a89dd22b75b61ead8f7083f251f'
 }
 
 map_schema_hash2version = {
@@ -143,7 +147,8 @@ map_schema_hash2version = {
 	#'419e5225259c53dd36ad80d82066ff02': 19	# 19.0 only
 	#'9765373098b03fb208332498f34cd4b5': 19 # until 19.11
 	'57f009a159f55f77525cc0291e0c8b60': 19, # starting with 19.12
-	'baed1901ed4c2f272b56c8cb2c6d88e8': 20
+	'baed1901ed4c2f272b56c8cb2c6d88e8': 20,
+	'e6a51a89dd22b75b61ead8f7083f251f': 21
 }
 
 map_client_branch2required_db_version = {
@@ -160,7 +165,8 @@ map_client_branch2required_db_version = {
 	u'1.2': 17,
 	u'1.3': 18,
 	u'1.4': 19,
-	u'1.5': 20
+	u'1.5': 20,
+	u'1.6': 21
 }
 
 # get columns and data types for a given table
@@ -285,6 +291,7 @@ def set_default_client_encoding(encoding = None):
 	_log.info('setting default client encoding from [%s] to [%s]' % (_default_client_encoding, str(encoding)))
 	_default_client_encoding = encoding
 	return True
+
 #---------------------------------------------------
 def set_default_client_timezone(timezone = None):
 
@@ -297,6 +304,7 @@ def set_default_client_timezone(timezone = None):
 	_sql_set_timezone = u'set timezone to %s'
 
 	return True
+
 #---------------------------------------------------
 def __validate_timezone(conn=None, timezone=None):
 
@@ -330,6 +338,7 @@ def __validate_timezone(conn=None, timezone=None):
 	conn.rollback()
 
 	return is_valid
+
 #---------------------------------------------------
 def __expand_timezone(conn=None, timezone=None):
 	"""some timezone defs are abbreviations so try to expand
@@ -353,7 +362,7 @@ where
 		curs.execute(cmd, args)
 		rows = curs.fetchall()
 		if len(rows) > 0:
-			result = rows[0][0]
+			result = rows[0]['name']
 			_log.debug(u'[%s] maps to [%s]', timezone, result)
 	except:
 		_log.exception(u'cannot expand timezone abbreviation [%s]', timezone)
@@ -362,6 +371,7 @@ where
 	conn.rollback()
 
 	return result
+
 #---------------------------------------------------
 def __detect_client_timezone(conn=None):
 	"""This is run on the very first connection."""
@@ -408,6 +418,7 @@ def __detect_client_timezone(conn=None):
 		_sql_set_timezone = u"set time zone interval %s hour to minute"
 
 	_log.info('client system time zone detected as equivalent to [%s]', _default_client_timezone)
+
 # =======================================================================
 # login API
 # =======================================================================
@@ -419,7 +430,7 @@ def __request_login_params_tui():
 	print "\nPlease enter the required login parameters:"
 	try:
 		login.host = prompted_input(prompt = "host ('' = non-TCP/IP)", default = '')
-		login.database = prompted_input(prompt = "database", default = 'gnumed_v20')
+		login.database = prompted_input(prompt = "database", default = default_database)
 		login.user = prompted_input(prompt = "user name", default = '')
 		tmp = 'password for "%s" (not shown): ' % login.user
 		login.password = getpass.getpass(tmp)
@@ -440,7 +451,7 @@ def __request_login_params_gui_wx():
 	# OK, wxPython was already loaded. But has the main Application instance
 	# been initialized yet ? if not, the exception will kick us out
 	if wx.GetApp() is None:
-		raise gmExceptions.NoGuiError(_("The wxPython GUI framework hasn't been initialized yet!"))
+		raise AssertionError(_("The wxPython GUI framework hasn't been initialized yet!"))
 
 	# Let's launch the login dialog
 	# if wx was not initialized /no main App loop, an exception should be raised anyway
@@ -465,7 +476,7 @@ def request_login_params():
 	# are we inside X ?
 	# (if we aren't wxGTK will crash hard at
 	# C-level with "can't open Display")
-	if os.environ.has_key('DISPLAY'):
+	if u'DISPLAY' in os.environ:
 		# try wxPython GUI
 		try: return __request_login_params_gui_wx()
 		except: pass
@@ -498,11 +509,13 @@ def make_psycopg2_dsn(database=None, host=None, port=5432, user=None, password=N
 	dsn_parts.append('sslmode=prefer')
 
 	return ' '.join(dsn_parts)
+
 # ------------------------------------------------------
 def get_default_login():
 	# make sure we do have a login
 	get_default_dsn()
 	return _default_login
+
 # ------------------------------------------------------
 def get_default_dsn():
 	global _default_dsn
@@ -513,6 +526,7 @@ def get_default_dsn():
 	set_default_login(login=login)
 
 	return _default_dsn
+
 # ------------------------------------------------------
 def set_default_login(login=None):
 	if login is None:
@@ -557,8 +571,9 @@ def log_auth_environment():
 				_log.debug('$PGPASSFILE=%s exists', pgpass_var)
 			else:
 				_log.debug('$PGPASSFILE=%s not found')
-	except StandardError:
+	except Exception:
 		_log.exception('cannot detect .pgpass and or $PGPASSFILE')
+
 # =======================================================================
 # netadata API
 # =======================================================================
@@ -790,7 +805,7 @@ def get_col_indices(cursor = None):
 		# a query like "select 1,2;" will return two columns of the same name !
 		# hence adjust to that, note, however, that dict-style access won't work
 		# on results of such queries ...
-		if col_indices.has_key(col_name):
+		if col_name in col_indices:
 			col_name = '%s_%s' % (col_name, col_index)
 		col_indices[col_name] = col_index
 		col_index += 1
@@ -825,7 +840,7 @@ def get_col_names(link_obj=None, schema='public', table=None):
 # i18n functions
 #------------------------------------------------------------------------
 def export_translations_from_database(filename=None):
-	tx_file = codecs.open(filename, 'wb', 'utf8')
+	tx_file = io.open(filename, mode = 'wt', encoding = 'utf8')
 	tx_file.write(u'-- GNUmed database string translations exported %s\n' % gmDateTime.pydt_now_here().strftime('%Y-%m-%d %H:%M'))
 	tx_file.write(u'-- - contains translations for each of [%s]\n' % u', '.join(get_translation_languages()))
 	tx_file.write(u'-- - user database language is set to [%s]\n\n' % get_current_user_language())
@@ -850,6 +865,7 @@ def export_translations_from_database(filename=None):
 	tx_file.close()
 
 	return True
+
 #------------------------------------------------------------------------
 def delete_translation_from_database(link_obj=None, language=None, original=None):
 	cmd = u'DELETE FROM i18n.translations WHERE lang = %(lang)s AND orig = %(orig)s'
@@ -1086,6 +1102,7 @@ def bytea2file(data_query=None, filename=None, chunk_size=0, data_size=None, dat
 	result = bytea2file_object(data_query=data_query, file_obj=outfile, chunk_size=chunk_size, data_size=data_size, data_size_query=data_size_query)
 	outfile.close()
 	return result
+
 #------------------------------------------------------------------------
 def bytea2file_object(data_query=None, file_obj=None, chunk_size=0, data_size=None, data_size_query=None):
 	"""Store data from a bytea field into a file.
@@ -1184,6 +1201,7 @@ def bytea2file_object(data_query=None, file_obj=None, chunk_size=0, data_size=No
 
 	conn.rollback()
 	return True
+
 #------------------------------------------------------------------------
 def file2bytea(query=None, filename=None, args=None, conn=None, file_md5=None):
 	"""Store data from a file into a bytea field.
@@ -1234,6 +1252,218 @@ def file2bytea(query=None, filename=None, args=None, conn=None, file_md5=None):
 		conn.close()
 
 	return success_status
+
+#------------------------------------------------------------------------
+def file2lo(filename=None, conn=None, check_md5=False):
+	# 1 GB limit unless 64 bit Python build ...
+	file_size = os.path.getsize(filename)
+	if file_size > (1024 * 1024) * 1024:
+		_log.debug(u'file size of [%s] > 1 GB, supposedly not supported by psycopg2 large objects (but seems to work anyway ?)', file_size)
+#		return -1
+
+	if conn is None:
+		conn = get_raw_connection(readonly = False)
+		close_conn = conn.close
+	else:
+		close_conn = __noop
+	_log.debug(u'[%s] -> large object', filename)
+
+	# insert the data
+	lo = conn.lobject(0, 'w', 0, filename)
+	lo_oid = lo.oid
+	lo.close()
+	_log.debug('large object OID: %s', lo_oid)
+
+	# verify
+	if file_md5 is None:
+		conn.commit()
+		close_conn()
+		return lo_oid
+	cmd = u'SELECT md5(lo_get(%(loid)s::oid))'
+	args = {'loid': lo_oid}
+	rows, idx = run_ro_queries(link_obj = conn, queries = [{'cmd': cmd, 'args': args}])
+	db_md5 = rows[0][0]
+	if file_md5 == db_md5:
+		conn.commit()
+		close_conn()
+		_log.debug('MD5 sums of data file and database large object match: [file::%s] = [DB::%s]', file_md5, db_md5)
+		return lo_oid
+	conn.rollback()
+	close_conn()
+	_log.error('MD5 sums of data file and database large object [%s] do not match: [file::%s] <> [DB::%s]', lo_oid, file_md5, db_md5)
+	return -1
+
+#------------------------------------------------------------------------
+def file2bytea_lo(filename=None, conn=None, file_md5=None):
+	# 1 GB limit unless 64 bit Python build ...
+	file_size = os.path.getsize(filename)
+	if file_size > (1024 * 1024) * 1024:
+		_log.debug(u'file size of [%s] > 1 GB, supposedly not supported by psycopg2 large objects (but seems to work anyway ?)', file_size)
+#		return -1
+
+	if conn is None:
+		conn = get_raw_connection(readonly = False)
+		close_conn = conn.close
+	else:
+		close_conn = __noop
+	_log.debug(u'[%s] -> large object', filename)
+
+	# insert the data
+	lo = conn.lobject(0, 'w', 0, filename)
+	lo_oid = lo.oid
+	lo.close()
+	_log.debug('large object OID: %s', lo_oid)
+
+	# verify
+	if file_md5 is None:
+		conn.commit()
+		close_conn()
+		return lo_oid
+	cmd = u'SELECT md5(lo_get(%(loid)s::oid))'
+	args = {'loid': lo_oid}
+	rows, idx = run_ro_queries(link_obj = conn, queries = [{'cmd': cmd, 'args': args}])
+	db_md5 = rows[0][0]
+	if file_md5 == db_md5:
+		conn.commit()
+		close_conn()
+		_log.debug('MD5 sums of data file and database large object match: [file::%s] = [DB::%s]', file_md5, db_md5)
+		return lo_oid
+	conn.rollback()
+	close_conn()
+	_log.error('MD5 sums of data file and database large object [%s] do not match: [file::%s] <> [DB::%s]', lo_oid, file_md5, db_md5)
+	return -1
+
+#------------------------------------------------------------------------
+def file2bytea_copy_from(table=None, columns=None, filename=None, conn=None, md5_query=None, file_md5=None):
+	# md5_query: dict{'cmd': ..., 'args': ...}
+
+	# UNTESTED
+
+	chunk_size = 32 * (1024 * 1024)
+	_log.debug('[%s] (%s bytes) --(%s bytes)-> %s(%s)', filename, os.path.getsize(filename), chunk_size, table, columns)
+	if conn is None:
+		conn = get_raw_connection(readonly = False)
+		close_conn = True
+	else:
+		close_conn = False
+	curs = conn.cursor()
+	# write
+	infile = file(filename, "rb")
+	curs.copy_from(infile, table, size = chunk_size, columns = columns)
+	infile.close()
+	curs.close()
+	if None in [file_md5, md5_query]:
+		conn.commit()
+		close_conn()
+		return True
+	# verify
+	rows, idx = run_ro_queries(link_obj = conn, queries = [md5_query])
+	db_md5 = rows[0][0]
+	if file_md5 == db_md5:
+		conn.commit()
+		close_conn()
+		_log.debug('MD5 sums of data file and database BYTEA field match: [file::%s] = [DB::%s]', file_md5, db_md5)
+		return True
+	close_conn()
+	_log.error('MD5 sums of data file and database BYTEA field do not match: [file::%s] <> [DB::%s]', file_md5, db_md5)
+	return False
+
+#------------------------------------------------------------------------
+def file2bytea_overlay(query=None, args=None, filename=None, conn=None, md5_query=None, file_md5=None):
+	"""Store data from a file into a bytea field.
+
+	The query must:
+	- 'cmd' must be in unicode
+	- 'cmd' must contain a format spec identifying the row (eg
+	  a primary key) matching <args> if it is an UPDATE
+	- 'cmd' must contain "... SET ... <some_bytea_field> = OVERLAY(some_bytea_field PLACING %(data)s::bytea FROM %(start)s FOR %(size)s) ..."
+	- 'args' must be a dict matching 'cmd'
+
+	The query CAN return the MD5 of the inserted data:
+		RETURNING md5(<field>) AS md5
+	in which case it will compare it to the md5
+	of the file.
+
+	UPDATE
+		the_table
+	SET
+		bytea_field = OVERLAY (
+			coalesce(bytea_field, '':bytea),
+			PLACING
+				%(data)s::bytea
+			FROM
+				%(start)s
+			FOR
+				%(size)s
+		)
+	WHERE
+		primary_key = pk_value
+
+	SELECT md5(bytea_field) FROM the_table WHERE primary_key = pk_value
+	"""
+	chunk_size = 32 * (1024 * 1024)
+	file_size = os.path.getsize(filename)
+	if file_size <= chunk_size:
+		chunk_size = file_size
+	needed_chunks, remainder = divmod(file_size, chunk_size)
+	_log.debug('file data: %s bytes, chunks: %s, chunk size: %s bytes, remainder: %s bytes', file_size, needed_chunks, chunk_size, remainder)
+
+	if conn is None:
+		conn = get_raw_connection(readonly = False)
+		close_conn = conn.close
+	else:
+		close_conn = __noop
+
+	infile = file(filename, "rb")
+	# write chunks
+	for chunk_id in range(needed_chunks):
+		chunk_start = (chunk_id * chunk_size) + 1
+		args['start'] = chunk_start
+		args['size'] = chunk_size
+		data_as_byte_string = infile.read(chunk_size)
+		args['data'] = buffer(data_as_byte_string)
+		del(data_as_byte_string)
+		try:
+			rows, idx = run_rw_queries(link_obj = conn, queries = [{'cmd': query, 'args': args}], end_tx = False, return_data = False)
+		except StandardError:
+			_log.exception('cannot write chunk [%s/%s] of size [%s], try decreasing chunk size', chunk_id+1, needed_chunks, chunk_size)
+			conn.rollback()
+			close_conn()
+			infile.close()
+			raise
+	# write remainder
+	if remainder > 0:
+		chunk_start = (needed_chunks * chunk_size) + 1
+		args['start'] = chunk_start
+		args['size'] = remainder
+		data_as_byte_string = infile.read(remainder)
+		args['data'] = buffer(data_as_byte_string)
+		del(data_as_byte_string)
+		try:
+			rows, idx = run_rw_queries(link_obj = conn, queries = [{'cmd': query, 'args': args}], end_tx = False, return_data = False)
+		except StandardError:
+			_log.error('cannot retrieve remaining [%s] bytes' % remainder)
+			conn.rollback()
+			close_conn()
+			infile.close()
+			raise
+	infile.close()
+	if None in [file_md5, md5_query]:
+		conn.commit()
+		close_conn()
+		return True
+	# verify
+	rows, idx = run_ro_queries(link_obj = conn, queries = [{'cmd': md5_query, 'args': args}])
+	db_md5 = rows[0][0]
+	if file_md5 == db_md5:
+		conn.commit()
+		close_conn()
+		_log.debug('MD5 sums of data file and database BYTEA field match: [file::%s] = [DB::%s]', file_md5, db_md5)
+		return True
+	close_conn()
+	_log.error('MD5 sums of data file and database BYTEA field do not match: [file::%s] <> [DB::%s]', file_md5, db_md5)
+	return False
+
 #------------------------------------------------------------------------
 def sanitize_pg_regex(expression=None, escape_all=False):
 	"""Escape input for use in a PostgreSQL regular expression.
@@ -1260,6 +1490,60 @@ def sanitize_pg_regex(expression=None, escape_all=False):
 			'*', '\*'
 		)
 		#']', '\]',			# not needed
+
+#------------------------------------------------------------------------
+def capture_cursor_state(cursor=None):
+	conn = cursor.connection
+
+	tx_status = conn.get_transaction_status()
+	if tx_status in [ psycopg2.extensions.TRANSACTION_STATUS_INERROR, psycopg2.extensions.TRANSACTION_STATUS_UNKNOWN ]:
+		isolation_level = u'tx aborted or unknown, cannot retrieve'
+	else:
+		isolation_level = conn.isolation_level
+
+	txt = u"""Link state:
+Cursor
+  identity: %s; name: %s
+  closed: %s; scrollable: %s; with hold: %s; arraysize: %s; itersize: %s;
+  last rowcount: %s; rownumber: %s; lastrowid (OID): %s;
+  last description: %s
+  statusmessage: %s
+Connection
+  identity: %s; backend pid: %s; protocol version: %s;
+  closed: %s; autocommit: %s; isolation level: %s; encoding: %s; async: %s;
+  TX status: %s; CX status: %s; executing async op: %s;
+Query
+  %s
+""" % (
+		id(cursor),
+		cursor.name,
+		cursor.closed,
+		cursor.scrollable,
+		cursor.withhold,
+		cursor.arraysize,
+		cursor.itersize,
+		cursor.rowcount,
+		cursor.rownumber,
+		cursor.lastrowid,
+		cursor.description,
+		cursor.statusmessage,
+
+		id(conn),
+		conn.get_backend_pid(),
+		conn.protocol_version,
+		conn.closed,
+		conn.autocommit,
+		isolation_level,
+		conn.encoding,
+		conn.async,
+		tx_status,
+		conn.status,
+		conn.isexecuting(),
+
+		cursor.query,
+	)
+	return txt
+
 #------------------------------------------------------------------------
 def run_ro_queries(link_obj=None, queries=None, verbose=False, return_data=True, get_col_idx=False):
 	"""Run read-only queries.
@@ -1273,17 +1557,26 @@ def run_ro_queries(link_obj=None, queries=None, verbose=False, return_data=True,
 	"""
 	if isinstance(link_obj, dbapi._psycopg.cursor):
 		curs = link_obj
-		curs_close = __noop
-		tx_rollback = __noop
+		curs_close = lambda :1
+		tx_rollback = lambda :1
+		readonly_rollback_just_in_case = lambda :1
 	elif isinstance(link_obj, dbapi._psycopg.connection):
 		curs = link_obj.cursor()
 		curs_close = curs.close
 		tx_rollback = link_obj.rollback
+		if link_obj.autocommit is True:		# readonly connection ?
+			readonly_rollback_just_in_case = link_obj.rollback
+		else:
+			# do not rollback readonly queries on passed-in readwrite
+			# connections just in case because they may have already
+			# seen fully legitimate write action which would get lost
+			readonly_rollback_just_in_case = lambda :1
 	elif link_obj is None:
 		conn = get_connection(readonly=True, verbose=verbose)
 		curs = conn.cursor()
 		curs_close = curs.close
 		tx_rollback = conn.rollback
+		readonly_rollback_just_in_case = conn.rollback
 	else:
 		raise ValueError('link_obj must be cursor, connection or None but not [%s]' % link_obj)
 
@@ -1301,14 +1594,10 @@ def run_ro_queries(link_obj=None, queries=None, verbose=False, return_data=True,
 		try:
 			curs.execute(query['cmd'], args)
 			if verbose:
-				_log.debug('ran query: [%s]', curs.query)
-				if curs.statusmessage != u'':
-					_log.debug('PG status message: %s', curs.statusmessage)
-				_log.debug('cursor description: %s', str(curs.description))
+				_log.debug(capture_cursor_state(curs))
 		except dbapi.Error as pg_exc:
-			_log.error('query failed: [%s]', curs.query)
-			if curs.statusmessage != u'':
-				_log.error('PG status message: %s', curs.statusmessage)
+			_log.error('query failed in RO connection')
+			_log.error(capture_cursor_state(curs))
 			pg_exc = make_pg_exception_fields_unicode(pg_exc)
 			_log.error('PG error code: %s', pg_exc.pgcode)
 			if pg_exc.pgerror is not None:
@@ -1337,9 +1626,8 @@ def run_ro_queries(link_obj=None, queries=None, verbose=False, return_data=True,
 				)
 			raise
 		except:
-			_log.error('query failed: [%s]', curs.query)
-			if curs.statusmessage != u'':
-				_log.error('PG status message: %s', curs.statusmessage)
+			_log.exception('query failed in RO connection')
+			_log.error(capture_cursor_state(curs))
 			try:
 				curs_close()
 			except dbapi.InterfaceError:
@@ -1358,8 +1646,9 @@ def run_ro_queries(link_obj=None, queries=None, verbose=False, return_data=True,
 			col_idx = get_col_indices(curs)
 
 	curs_close()
-	tx_rollback()		# rollback just so that we don't stay IDLE IN TRANSACTION forever
+	readonly_rollback_just_in_case()
 	return (data, col_idx)
+
 #------------------------------------------------------------------------
 def run_rw_queries(link_obj=None, queries=None, end_tx=False, return_data=None, get_col_idx=False, verbose=False):
 	"""Convenience function for running a transaction
@@ -1408,21 +1697,23 @@ def run_rw_queries(link_obj=None, queries=None, end_tx=False, return_data=None,
 			* for <index> see <get_col_idx>
 	"""
 	if isinstance(link_obj, dbapi._psycopg.cursor):
-		conn_close = __noop
-		conn_commit = __noop
-		tx_rollback = __noop
+		conn_close = lambda :1
+		conn_commit = lambda :1
+		tx_rollback = lambda :1
 		curs = link_obj
-		curs_close = __noop
+		curs_close = lambda :1
+		notices_accessor = curs.connection
 	elif isinstance(link_obj, dbapi._psycopg.connection):
-		conn_close = __noop
+		conn_close = lambda :1
 		if end_tx:
 			conn_commit = link_obj.commit
 			tx_rollback = link_obj.rollback
 		else:
-			conn_commit = __noop
-			tx_rollback = __noop
+			conn_commit = lambda :1
+			tx_rollback = lambda :1
 		curs = link_obj.cursor()
 		curs_close = curs.close
+		notices_accessor = link_obj
 	elif link_obj is None:
 		conn = get_connection(readonly=False)
 		conn_close = conn.close
@@ -1430,6 +1721,7 @@ def run_rw_queries(link_obj=None, queries=None, end_tx=False, return_data=None,
 		tx_rollback = conn.rollback
 		curs = conn.cursor()
 		curs_close = curs.close
+		notices_accessor = conn
 	else:
 		raise ValueError('link_obj must be cursor, connection or None but not [%s]' % link_obj)
 
@@ -1443,20 +1735,23 @@ def run_rw_queries(link_obj=None, queries=None, end_tx=False, return_data=None,
 			args = None
 		try:
 			curs.execute(query['cmd'], args)
+			if verbose:
+				_log.debug(capture_cursor_state(curs))
+			for notice in notices_accessor.notices:
+				_log.debug(notice.strip(u'\n').strip(u'\r'))
+			del notices_accessor.notices[:]
+		# DB related exceptions
 		except dbapi.Error as pg_exc:
-			_log.error('RW query failed: [%s]', curs.query)
-			if curs.statusmessage != u'':
-				_log.error('PG status message: %s', curs.statusmessage)
+			_log.error('query failed in RW connection')
+			_log.error(capture_cursor_state(curs))
+			for notice in notices_accessor.notices:
+				_log.error(notice.strip(u'\n').strip(u'\r'))
+			del notices_accessor.notices[:]
 			pg_exc = make_pg_exception_fields_unicode(pg_exc)
 			_log.error(u'PG error code: %s', pg_exc.pgcode)
 			if pg_exc.pgerror is not None:
 				_log.error(u'PG error message: %s', pg_exc.u_pgerror)
-			try:
-				curs_close()
-				tx_rollback()			# just for good measure
-				conn_close()
-			except dbapi.InterfaceError:
-				_log.exception('cannot cleanup')
+			# privilege problem
 			if pg_exc.pgcode == sql_error_codes.INSUFFICIENT_PRIVILEGE:
 				details = u'Query: [%s]' % curs.query.strip().strip(u'\n').strip().strip(u'\n')
 				if curs.statusmessage != u'':
@@ -1468,15 +1763,34 @@ def run_rw_queries(link_obj=None, queries=None, end_tx=False, return_data=None,
 					msg = u'[%s]' % pg_exc.pgcode
 				else:
 					msg = u'[%s]: %s' % (pg_exc.pgcode, pg_exc.u_pgerror)
+				try:
+					curs_close()
+					tx_rollback()			# just for good measure
+					conn_close()
+				except dbapi.InterfaceError:
+					_log.exception('cannot cleanup')
 				raise gmExceptions.AccessDenied (
 					msg,
 					source = u'PostgreSQL',
 					code = pg_exc.pgcode,
 					details = details
 				)
+			# other problem
+			gmLog2.log_stack_trace()
+			try:
+				curs_close()
+				tx_rollback()			# just for good measure
+				conn_close()
+			except dbapi.InterfaceError:
+				_log.exception('cannot cleanup')
 			raise
+		# other exception
 		except:
-			_log.exception('error running RW query')
+			_log.exception('error running query in RW connection')
+			_log.error(capture_cursor_state(curs))
+			for notice in notices_accessor.notices:
+				_log.debug(notice.strip(u'\n').strip(u'\r'))
+			del notices_accessor.notices[:]
 			gmLog2.log_stack_trace()
 			try:
 				curs_close()
@@ -1484,7 +1798,6 @@ def run_rw_queries(link_obj=None, queries=None, end_tx=False, return_data=None,
 				conn_close()
 			except dbapi.InterfaceError:
 				_log.exception('cannot cleanup')
-				raise
 			raise
 
 	data = None
@@ -1511,6 +1824,7 @@ def run_rw_queries(link_obj=None, queries=None, end_tx=False, return_data=None,
 	conn_close()
 
 	return (data, col_idx)
+
 #------------------------------------------------------------------------
 def run_insert(link_obj=None, schema=None, table=None, values=None, returning=None, end_tx=False, get_col_idx=False, verbose=False):
 	"""Generates SQL for an INSERT query.
@@ -1554,6 +1868,7 @@ def run_insert(link_obj=None, schema=None, table=None, values=None, returning=No
 		get_col_idx = get_col_idx,
 		verbose = verbose
 	)
+
 # =======================================================================
 # connection handling API
 # -----------------------------------------------------------------------
@@ -1585,6 +1900,7 @@ class cConnectionPool(psycopg2.pool.PersistentConnectionPool):
 		for conn_key in self._used.keys():
 			_log.debug('closing pooled database connection, pool key: %s, backend PID: %s', conn_key, self._used[conn_key].get_backend_pid())
 			self._used[conn_key].original_close()
+
 # -----------------------------------------------------------------------
 def get_raw_connection(dsn=None, verbose=False, readonly=True):
 	"""Get a raw, unadorned connection.
@@ -1603,6 +1919,7 @@ def get_raw_connection(dsn=None, verbose=False, readonly=True):
 
 	try:
 		conn = dbapi.connect(dsn=dsn, connection_factory=psycopg2.extras.DictConnection)
+		#conn = dbapi.connect(dsn=dsn, cursor_factory=psycopg2.extras.RealDictCursor)
 	except dbapi.OperationalError, e:
 
 		t, v, tb = sys.exc_info()
@@ -1866,7 +2183,8 @@ def sanity_check_database_settings():
 		#u'regex_flavor': [u'advanced', u'query breakage', False],					# 9.0 doesn't support this anymore, default now advanced anyway
 		u'synchronous_commit': [u'on', u'data loss/corruption', False],
 		u'sql_inheritance': [u'on', u'query breakage, data loss/corruption', True],
-		u'ignore_checksum_failure': [u'off', u'data loss/corruption', False]		# starting with PG 9.3
+		u'ignore_checksum_failure': [u'off', u'data loss/corruption', False],		# starting with PG 9.3
+		u'track_commit_timestamp': [u'on', u'suboptimal auditing', False]			# starting with PG 9.3
 	}
 
 	from Gnumed.pycommon import gmCfg2
@@ -1930,7 +2248,7 @@ def _log_PG_settings(curs=None):
 		_log.error(u'cannot log PG settings (>>>show all<<< did not return rows)')
 		return False
 	for setting in settings:
-		_log.debug(u'PG option [%s]: %s', setting[0], setting[1])
+		_log.debug(u'PG option [%s]: %s', setting['name'], setting['setting'])
 
 	try:
 		curs.execute(u'select pg_available_extensions()')
@@ -1942,7 +2260,7 @@ def _log_PG_settings(curs=None):
 		_log.error(u'no PG extensions available')
 		return False
 	for ext in extensions:
-		_log.debug(u'PG extension: %s', ext[0])
+		_log.debug(u'PG extension: %s', ext['pg_available_extensions'])
 
 	return True
 #========================================================================
@@ -2068,22 +2386,99 @@ if __name__ == "__main__":
 	if sys.argv[1] != 'test':
 		sys.exit()
 
+	from Gnumed.pycommon.gmTools import file2md5
+
 	logging.basicConfig(level=logging.DEBUG)
+
 	#--------------------------------------------------------------------
 	def test_file2bytea():
 		run_rw_queries(queries = [
+			{'cmd': u'drop table if exists test_bytea'},
 			{'cmd': u'create table test_bytea (data bytea)'}
 		])
 
-		cmd = u'insert into test_bytea values (%(data)s::bytea)'
 		try:
-			file2bytea(query = cmd, filename = sys.argv[2])
+			file2bytea(query = u'insert into test_bytea values (%(data)s::bytea) returning md5(data) as md5', filename = sys.argv[2], file_md5 = file2md5(sys.argv[2], True))
 		except:
 			_log.exception('error')
 
 		run_rw_queries(queries = [
 			{'cmd': u'drop table test_bytea'}
 		])
+
+	#--------------------------------------------------------------------
+	def test_file2bytea_lo():
+		lo_oid = file2bytea_lo (
+			filename = sys.argv[2]
+			#, file_md5 = file2md5(sys.argv[2], True)
+		)
+		print lo_oid
+#		if lo_oid != -1:
+#			run_rw_queries(queries = [
+#				{'cmd': u'select lo_unlink(%(loid)s::oid)', 'args': {'loid': lo_oid}}
+#			])
+
+	#--------------------------------------------------------------------
+	def test_file2bytea_copy_from():
+
+		run_rw_queries(queries = [
+			{'cmd': u'drop table if exists test_bytea'},
+			{'cmd': u'create table test_bytea (pk serial primary key, data bytea)'},
+			{'cmd': u"insert into test_bytea (data) values (NULL::bytea)"}
+		])
+
+		md5_query = {
+			'cmd': u'select md5(data) AS md5 FROM test_bytea WHERE pk = %(pk)s',
+			'args': {'pk': 1}
+		}
+
+		file2bytea_copy_from (
+			table = u'test_bytea',
+			columns = [u'data'],
+			filename = sys.argv[2],
+			md5_query = md5_query,
+			file_md5 = file2md5(sys.argv[2], True)
+		)
+
+		run_rw_queries(queries = [
+			{'cmd': u'drop table if exists test_bytea'}
+		])
+
+	#--------------------------------------------------------------------
+	def test_file2bytea_overlay():
+
+		run_rw_queries(queries = [
+			{'cmd': u'drop table if exists test_bytea'},
+			{'cmd': u'create table test_bytea (pk serial primary key, data bytea)'},
+			{'cmd': u"insert into test_bytea (data) values (NULL::bytea)"}
+		])
+
+		cmd = u"""
+		update test_bytea
+		set data = overlay (
+			coalesce(data, ''::bytea)
+			placing %(data)s::bytea
+			from %(start)s
+			for %(size)s
+		)
+		where
+			pk > %(pk)s
+		"""
+		md5_cmd = u'select md5(data) from test_bytea'
+		args = {'pk': 0}
+		file2bytea_overlay (
+			query = cmd,
+			args = args,
+			filename = sys.argv[2],
+			conn = None,
+			md5_query = md5_cmd,
+			file_md5 = file2md5(sys.argv[2], True)
+		)
+
+		run_rw_queries(queries = [
+			{'cmd': u'drop table test_bytea'}
+		])
+
 	#--------------------------------------------------------------------
 	def test_get_connection():
 		print "testing get_connection()"
@@ -2452,9 +2847,12 @@ SELECT to_timestamp (foofoo,'YYMMDD.HH24MI') FROM (
 	def test_get_index_name():
 		print get_index_name(indexed_table = 'clin.vaccination', indexed_column = 'fk_episode')
 
+	#--------------------------------------------------------------------
+	def test_faulty_SQL():
+		run_rw_queries(queries = [{'cmd': u'SELEC 1'}])
+
 	#--------------------------------------------------------------------
 	# run tests
-	#test_file2bytea()
 	#test_get_connection()
 	#test_exceptions()
 	#test_ro_queries()
@@ -2467,12 +2865,17 @@ SELECT to_timestamp (foofoo,'YYMMDD.HH24MI') FROM (
 	#test_sanity_check_time_skew()
 	#test_get_foreign_key_details()
 	#test_get_foreign_key_names()
-	test_get_index_name()
+	#test_get_index_name()
 	#test_set_user_language()
 	#test_get_schema_revision_history()
 	#test_run_query()
 	#test_schema_exists()
 	#test_get_foreign_key_names()
 	#test_row_locks()
+	#test_file2bytea()
+	#test_file2bytea_overlay()
+	#test_file2bytea_copy_from()
+	#test_file2bytea_lo()
+	test_faulty_SQL()
 
 # ======================================================================
