@@ -37,6 +37,9 @@ import timelinelib.time.timeline as timeline
 
 class GregorianTimeType(TimeType):
 
+    def __init__(self):
+        self.major_strip_is_decade = False
+
     def __eq__(self, other):
         return isinstance(other, GregorianTimeType)
 
@@ -65,7 +68,7 @@ class GregorianTimeType(TimeType):
     def get_navigation_functions(self):
         return [
             (_("Go to &Today\tCtrl+T"), go_to_today_fn),
-            (_("Go to D&ate...\tCtrl+G"), go_to_date_fn),
+            (_("Go to &Date...\tCtrl+G"), go_to_date_fn),
             ("SEP", None),
             (_("Backward\tPgUp"), backward_fn),
             (_("Forward\tPgDn"), forward_fn),
@@ -113,8 +116,8 @@ class GregorianTimeType(TimeType):
 
     def format_delta(self, delta):
         days = delta.get_days()
-        hours = delta.get_hours() 
-        minutes = delta.get_minutes() 
+        hours = delta.get_hours()
+        minutes = delta.get_minutes()
         collector = []
         if days == 1:
             collector.append(u"1 %s" % _("day"))
@@ -146,6 +149,7 @@ class GregorianTimeType(TimeType):
         """
         day_period = TimePeriod(self, timeline.Time(0, 0), timeline.Time(1, 0))
         one_day_width = metrics.calc_exact_width(day_period)
+        self.major_strip_is_decade = False
         if one_day_width > 600:
             return (StripDay(), StripHour())
         elif one_day_width > 45:
@@ -155,6 +159,7 @@ class GregorianTimeType(TimeType):
         elif one_day_width > 1.5:
             return (StripYear(), StripMonth())
         elif one_day_width > 0.12:
+            self.major_strip_is_decade = True
             return (StripDecade(), StripYear())
         elif one_day_width > 0.012:
             return (StripCentury(), StripDecade())
@@ -231,7 +236,13 @@ class GregorianTimeType(TimeType):
         return s1 == s2
 
     def adjust_for_bc_years(self, time):
-        return time
+        gregorian_time = gregorian.from_time(time)
+        if self.major_strip_is_decade:
+            if gregorian_time.year < 0:
+                gregorian_time.year += 2
+            elif gregorian_time.year < 10:
+                gregorian_time.year += 1
+        return gregorian_time.to_time()
 
 
 def go_to_today_fn(main_frame, current_period, navigation_fn):
@@ -263,12 +274,29 @@ def _move_page_smart(current_period, navigation_fn, direction):
 
 
 def _whole_number_of_years(period):
-    start, end = period.start_time, period.end_time
-    year_diff = _calculate_year_diff(period)
-    old_start = gregorian.from_time(start)
-    new_start = old_start.replace(year=old_start.year + year_diff).to_time()
-    whole_years = new_start == end
-    return whole_years and year_diff > 0
+    """
+    >>> from specs.utils import gregorian_period
+    >>> from specs.utils import install_gettext_in_builtin_namespace
+
+    >>> _whole_number_of_years(gregorian_period("1 Jan 2013", "1 Jan 2014"))
+    True
+    >>> install_gettext_in_builtin_namespace() # Needed because REPL replaces _
+
+    >>> _whole_number_of_years(gregorian_period("1 Jan 2013", "1 Jan 2015"))
+    True
+    >>> install_gettext_in_builtin_namespace() # Needed because REPL replaces _
+
+    >>> _whole_number_of_years(gregorian_period("1 Feb 2013", "1 Feb 2014"))
+    False
+    >>> install_gettext_in_builtin_namespace() # Needed because REPL replaces _
+
+    >>> _whole_number_of_years(gregorian_period("1 Jan 2013", "1 Feb 2014"))
+    False
+    >>> install_gettext_in_builtin_namespace() # Needed because REPL replaces _
+    """
+    return (gregorian.from_time(period.start_time).is_jan_first() and
+            gregorian.from_time(period.end_time).is_jan_first() and
+            _calculate_year_diff(period) > 0)
 
 
 def _move_page_years(curret_period, navigation_fn, direction):
@@ -295,16 +323,38 @@ def _move_page_years(curret_period, navigation_fn, direction):
 
 
 def _calculate_year_diff(period):
-    return gregorian.from_time(period.end_time).year - gregorian.from_time(period.start_time).year
+    return (gregorian.from_time(period.end_time).year -
+            gregorian.from_time(period.start_time).year)
 
 
 def _whole_number_of_months(period):
+    """
+    >>> from specs.utils import gregorian_period
+    >>> from specs.utils import install_gettext_in_builtin_namespace
+
+    >>> _whole_number_of_months(gregorian_period("1 Jan 2013", "1 Jan 2014"))
+    True
+    >>> install_gettext_in_builtin_namespace() # Needed because REPL replaces _
+
+    >>> _whole_number_of_months(gregorian_period("1 Jan 2013", "1 Mar 2014"))
+    True
+    >>> install_gettext_in_builtin_namespace() # Needed because REPL replaces _
+
+    >>> _whole_number_of_months(gregorian_period("2 Jan 2013", "2 Mar 2014"))
+    False
+    >>> install_gettext_in_builtin_namespace() # Needed because REPL replaces _
+
+    >>> _whole_number_of_months(gregorian_period("1 Jan 2013 12:00", "1 Mar 2014"))
+    False
+    >>> install_gettext_in_builtin_namespace() # Needed because REPL replaces _
+    """
     start, end = gregorian.from_time(period.start_time), gregorian.from_time(period.end_time)
     start_months = start.year * 12 + start.month
     end_months = end.year * 12 + end.month
     month_diff = end_months - start_months
-    whole_months = start.day == 1 and end.day == 1
-    return whole_months and month_diff > 0
+    return (start.is_first_of_month() and
+            end.is_first_of_month() and
+            month_diff > 0)
 
 
 def _move_page_months(curret_period, navigation_fn, direction):
@@ -402,14 +452,14 @@ def fit_millennium_fn(main_frame, current_period, navigation_fn):
     if mean.year > get_millenium_max_year():
         year = get_millenium_max_year()
     else:
-        year = max(get_min_year(), int(mean.year/1000)*1000)
+        year = max(get_min_year_containing_jan_1(), int(mean.year/1000)*1000)
     start = gregorian.from_date(year, 1, 1).to_time()
     end = gregorian.from_date(year + 1000, 1, 1).to_time()
     navigation_fn(lambda tp: tp.update(start, end))
 
 
-def get_min_year():
-    return gregorian.from_time(GregorianTimeType().get_min_time()[0]).year
+def get_min_year_containing_jan_1():
+    return gregorian.from_time(GregorianTimeType().get_min_time()[0]).year + 1
 
 
 def get_millenium_max_year():
@@ -425,7 +475,7 @@ def fit_century_fn(main_frame, current_period, navigation_fn):
     if mean.year > get_century_max_year():
         year = get_century_max_year()
     else:
-        year = max(get_min_year(), int(mean.year/100)*100)
+        year = max(get_min_year_containing_jan_1(), int(mean.year/100)*100)
     start = gregorian.from_date(year, 1, 1).to_time()
     end = gregorian.from_date(year + 100, 1, 1).to_time()
     navigation_fn(lambda tp: tp.update(start, end))
@@ -480,8 +530,11 @@ class StripCentury(Strip):
             # TODO: This only works for English. Possible to localize?
             time = gregorian.from_time(time)
             start_year = self._century_start_year(time.year)
-            next_start_year = start_year + 100
-            return str(next_start_year / 100) + " century"
+            century = (start_year + 100) / 100
+            if century <= 0:
+                return str(abs(century - 1)) + " century BC"
+            else:
+                return str(century) + " century"
         return ""
 
     def start(self, time):
@@ -516,7 +569,10 @@ class StripDecade(Strip):
         return gregorian_time.replace(year=gregorian_time.year+10).to_time()
 
     def _decade_start_year(self, year):
-        return (int(year) / 10) * 10
+        # The first start year must be to the left of the first visible
+        # year on the timeline in order to draw the first vertical decade
+        # line correctly. Therefore -10 in the calculation below
+        return (int(year) / 10) * 10 - 10
 
     def get_font(self, time_period):
         return get_default_font(8)
@@ -652,8 +708,8 @@ class StripWeekday(Strip):
                                     time.day,
                                     abbreviated_name_of_month(time.month),
                                     format_year(time.year))
-        return (abbreviated_name_of_weekday(time.get_day_of_week()) + 
-                " %s" % gregorian.from_time(time).day) 
+        return (abbreviated_name_of_weekday(time.get_day_of_week()) +
+                " %s" % gregorian.from_time(time).day)
 
     def start(self, time):
         gregorian_time = gregorian.from_time(time)
@@ -695,11 +751,10 @@ def format_year(year):
 
 
 def format_decade(start_year):
-    # TODO: This only works for English. Possible to localize?
-    if start_year <= 0:
-        return "%ds BC" % abs(start_year)
+    if start_year >= -10:
+        return str(start_year + 10) + "s"
     else:
-        return str(start_year) + "s"
+        return "%ds BC" % (abs(start_year) - 20)
 
 
 def move_period_num_days(period, num):
