@@ -7,7 +7,7 @@ __license__ = "GPL v2 or later"
 import sys
 import os.path
 import StringIO
-import codecs
+import io
 import logging
 
 
@@ -30,6 +30,8 @@ from Gnumed.business import gmEMRStructItems
 from Gnumed.business import gmPerson
 from Gnumed.business import gmSOAPimporter
 from Gnumed.business import gmPersonSearch
+from Gnumed.business import gmSoapDefs
+from Gnumed.business import gmClinicalRecord
 
 from Gnumed.wxpython import gmGuiHelpers
 from Gnumed.wxpython import gmEMRStructWidgets
@@ -38,9 +40,12 @@ from Gnumed.wxpython import gmSOAPWidgets
 from Gnumed.wxpython import gmAllergyWidgets
 from Gnumed.wxpython import gmDemographicsWidgets
 from Gnumed.wxpython import gmNarrativeWidgets
+from Gnumed.wxpython import gmNarrativeWorkflows
 from Gnumed.wxpython import gmPatSearchWidgets
 from Gnumed.wxpython import gmVaccWidgets
 from Gnumed.wxpython import gmFamilyHistoryWidgets
+from Gnumed.wxpython import gmFormWidgets
+from Gnumed.wxpython import gmTimer
 
 
 _log = logging.getLogger('gm.ui')
@@ -82,7 +87,7 @@ def export_emr_to_ascii(parent=None):
 
 	_log.debug('exporting EMR to [%s]', fname)
 
-	output_file = codecs.open(fname, 'wb', encoding='utf8', errors='replace')
+	output_file = io.open(fname, mode = 'wt', encoding = 'utf8', errors = 'replace')
 	exporter = gmPatientExporter.cEmrExport(patient = pat)
 	exporter.set_output_file(output_file)
 	exporter.dump_constraints()
@@ -93,6 +98,7 @@ def export_emr_to_ascii(parent=None):
 
 	gmDispatcher.send('statustext', msg = _('EMR successfully exported to file: %s') % fname, beep = False)
 	return fname
+
 #============================================================
 class cEMRTree(wx.TreeCtrl, treemixin.ExpansionState):
 	"""This wx.TreeCtrl derivative displays a tree view of a medical record."""
@@ -116,6 +122,7 @@ class cEMRTree(wx.TreeCtrl, treemixin.ExpansionState):
 
 		self.__make_popup_menus()
 		self.__register_events()
+
 	#--------------------------------------------------------
 	# external API
 	#--------------------------------------------------------
@@ -124,8 +131,13 @@ class cEMRTree(wx.TreeCtrl, treemixin.ExpansionState):
 
 	def _set_soap_display(self, soap_display=None):
 		self.__soap_display = soap_display
+		self.__soap_display_prop_font = soap_display.GetFont()
+		self.__soap_display_mono_font = wx.FontFromNativeInfo(self.__soap_display_prop_font.NativeFontInfo)
+		self.__soap_display_mono_font.SetFamily(wx.FONTFAMILY_TELETYPE)
+		self.__soap_display_mono_font.SetPointSize(self.__soap_display_prop_font.GetPointSize() - 2)
 
 	soap_display = property(_get_soap_display, _set_soap_display)
+
 	#--------------------------------------------------------
 	def _get_image_display(self):
 		return self.__img_display
@@ -134,11 +146,13 @@ class cEMRTree(wx.TreeCtrl, treemixin.ExpansionState):
 		self.__img_display = image_display
 
 	image_display = property(_get_image_display, _set_image_display)
+
 	#--------------------------------------------------------
 	def set_enable_display_mode_selection_callback(self, callback):
 		if not callable(callback):
 			raise ValueError('callback [%s] not callable' % callback)
 		self.__cb__enable_display_mode_selection = callback
+
 	#--------------------------------------------------------
 	def _set_edit_mode_selector(self, callback):
 		if callback is None:
@@ -148,6 +162,7 @@ class cEMRTree(wx.TreeCtrl, treemixin.ExpansionState):
 		self.__cb__select_edit_mode = callback
 
 	edit_mode_selector = property(lambda x:x, _set_edit_mode_selector)
+
 	#--------------------------------------------------------
 	def _set_soap_editor_adder(self, callback):
 		if callback is None:
@@ -157,6 +172,7 @@ class cEMRTree(wx.TreeCtrl, treemixin.ExpansionState):
 		self.__cb__add_soap_editor = callback
 
 	soap_editor_adder = property(lambda x:x, _set_soap_editor_adder)
+
 	#--------------------------------------------------------
 	# ExpansionState mixin API
 	#--------------------------------------------------------
@@ -190,6 +206,7 @@ class cEMRTree(wx.TreeCtrl, treemixin.ExpansionState):
 			return u'dummy node::%s' % self.__pat.ID
 		# root node == EMR level
 		return u'root node::%s' % self.__pat.ID
+
 	#--------------------------------------------------------
 	# internal helpers
 	#--------------------------------------------------------
@@ -208,10 +225,12 @@ class cEMRTree(wx.TreeCtrl, treemixin.ExpansionState):
 		gmDispatcher.connect(signal = 'clin.episode_mod_db', receiver = self._on_episode_mod_db)
 		gmDispatcher.connect(signal = 'clin.health_issue_mod_db', receiver = self._on_issue_mod_db)
 		gmDispatcher.connect(signal = 'clin.family_history_mod_db', receiver = self._on_issue_mod_db)
+
 	#--------------------------------------------------------
 	def clear_tree(self):
 		self.DeleteAllItems()
 		self.__expanded_nodes = None
+
 	#--------------------------------------------------------
 	def __populate_tree(self):
 		"""Updates EMR browser data."""
@@ -222,6 +241,7 @@ class cEMRTree(wx.TreeCtrl, treemixin.ExpansionState):
 
 		if self.__pat is None:
 			self.clear_tree()
+			self.__expanded_nodes = None
 			wx.EndBusyCursor()
 			return True
 
@@ -286,16 +306,18 @@ class cEMRTree(wx.TreeCtrl, treemixin.ExpansionState):
 			self.__root_tooltip = u' '
 
 		return root_item
+
 	#--------------------------------------------------------
 	def __update_text_for_selected_node(self):
 		"""Displays information for the selected tree node."""
 
 		if self.__soap_display is None:
-			self.__img_display.clear()
 			return
 
+		self.__soap_display.Clear()
+		self.__img_display.clear()
+
 		if self.__curr_node is None:
-			self.__img_display.clear()
 			return
 
 		if not self.__curr_node.IsOk():
@@ -319,18 +341,27 @@ class cEMRTree(wx.TreeCtrl, treemixin.ExpansionState):
 			self.__cb__enable_display_mode_selection(True)
 			txt = u'invalid SOAP display mode [%s]' % self.__soap_display_mode
 			if self.__soap_display_mode == u'details':
-				txt = node_data.format(left_margin=1, patient = self.__pat)
+				txt = node_data.format(left_margin = 1, patient = self.__pat)
+				font = self.__soap_display_prop_font
 			if self.__soap_display_mode == u'journal':
 				txt = node_data.format_as_journal(left_margin = 1)
+				font = self.__soap_display_prop_font
 			if self.__soap_display_mode == u'revisions':
 				txt = node_data.formatted_revision_history
-			self.__img_display.refresh (
-				document_folder = doc_folder,
-				episodes = [ epi['pk_episode'] for epi in node_data.episodes ]
-			)
+				font = self.__soap_display_mono_font
+			epis = node_data.episodes
+			if len(epis) > 0:
+				self.__img_display.refresh (
+					document_folder = doc_folder,
+					episodes = [ epi['pk_episode'] for epi in epis ]
+				)
+			self.__soap_display.SetFont(font)
+			self.__soap_display.WriteText(txt)
+			self.__soap_display.ShowPosition(0)
+			return
 
 		# unassociated episodes		# FIXME: turn into real dummy issue
-		elif isinstance(node_data, type({})):
+		if isinstance(node_data, type({})):
 			self.__cb__enable_display_mode_selection(True)
 			if self.__soap_display_mode == u'details':
 				txt = _('Pool of unassociated episodes "%s":\n') % node_data['description']
@@ -376,28 +407,38 @@ class cEMRTree(wx.TreeCtrl, treemixin.ExpansionState):
 					)
 					txt += u'\n'
 					txt += epi.format_as_journal(left_margin = 2)
-			self.__img_display.clear()
+			self.__soap_display.SetFont(self.__soap_display_prop_font)
+			self.__soap_display.WriteText(txt)
+			self.__soap_display.ShowPosition(0)
+			return
 
-		elif isinstance(node_data, gmEMRStructItems.cEpisode):
+		if isinstance(node_data, gmEMRStructItems.cEpisode):
 			self.__cb__enable_display_mode_selection(True)
 			txt = u'invalid SOAP display mode [%s]' % self.__soap_display_mode
 			if self.__soap_display_mode == u'details':
 				txt = node_data.format(left_margin = 1, patient = self.__pat)
+				font = self.__soap_display_prop_font
 			if self.__soap_display_mode == u'journal':
 				txt = node_data.format_as_journal(left_margin = 1)
+				font = self.__soap_display_prop_font
 			if self.__soap_display_mode == u'revisions':
 				txt = node_data.formatted_revision_history
+				font = self.__soap_display_mono_font
 			self.__img_display.refresh (
 				document_folder = doc_folder,
-				episodes = [node_data['pk_episode']]
+				episodes = [ node_data['pk_episode'] ]
 			)
+			self.__soap_display.SetFont(font)
+			self.__soap_display.WriteText(txt)
+			self.__soap_display.ShowPosition(0)
+			return
 
-		elif isinstance(node_data, gmEMRStructItems.cEncounter):
-			#self.__cb__enable_display_mode_selection(False)
+		if isinstance(node_data, gmEMRStructItems.cEncounter):
 			self.__cb__enable_display_mode_selection(True)
 			epi = self.GetPyData(self.GetItemParent(self.__curr_node))
 			if self.__soap_display_mode == u'revisions':
 				txt = node_data.formatted_revision_history
+				font = self.__soap_display_mono_font
 			else:
 				txt = node_data.format (
 					episodes = [epi['pk_episode']],
@@ -406,31 +447,37 @@ class cEMRTree(wx.TreeCtrl, treemixin.ExpansionState):
 					patient = self.__pat,
 					with_co_encountlet_hints = True
 				)
+				font = self.__soap_display_prop_font
 			self.__img_display.refresh (
 				document_folder = doc_folder,
-				episodes = [epi['pk_episode']],
+				episodes = [ epi['pk_episode'] ],
 				encounter = node_data['pk_encounter']
 			)
+			self.__soap_display.SetFont(font)
+			self.__soap_display.WriteText(txt)
+			self.__soap_display.ShowPosition(0)
+			return
 
 		# root node == EMR level
+		self.__cb__enable_display_mode_selection(True)
+		if self.__soap_display_mode == u'details':
+			emr = self.__pat.get_emr()
+			txt = emr.format_summary()
 		else:
-			self.__cb__enable_display_mode_selection(True)
-			if self.__soap_display_mode == u'details':
-				emr = self.__pat.get_emr()
-				txt = emr.format_summary()
-			else:
-				txt = self.__pat.emr.format_as_journal(left_margin = 1, patient = self.__pat)
-			self.__img_display.clear()
-
-		self.__soap_display.Clear()
+			txt = self.__pat.emr.format_as_journal(left_margin = 1, patient = self.__pat)
+		self.__soap_display.SetFont(self.__soap_display_prop_font)
 		self.__soap_display.WriteText(txt)
 		self.__soap_display.ShowPosition(0)
+
 	#--------------------------------------------------------
 	def __make_popup_menus(self):
 
 		# - root node
 		self.__root_context_popup = wx.Menu(title = _('EMR Actions:'))
 
+		item = self.__root_context_popup.Append(-1, _('Print EMR'))
+		self.Bind(wx.EVT_MENU, self.__print_emr, item)
+
 		menu_id = wx.NewId()
 		self.__root_context_popup.AppendItem(wx.MenuItem(self.__root_context_popup, menu_id, _('Create health issue')))
 		wx.EVT_MENU(self.__root_context_popup, menu_id, self.__create_issue)
@@ -550,37 +597,45 @@ class cEMRTree(wx.TreeCtrl, treemixin.ExpansionState):
 
 		item = self.__enc_context_popup.Append(-1, _('Export for Medistar'))
 		self.Bind(wx.EVT_MENU, self.__export_encounter_for_medistar, item)
+
 	#--------------------------------------------------------
 	def __handle_root_context(self, pos=wx.DefaultPosition):
 		self.PopupMenu(self.__root_context_popup, pos)
+
 	#--------------------------------------------------------
 	def __handle_issue_context(self, pos=wx.DefaultPosition):
 #		self.__issue_context_popup.SetTitle(_('Episode %s') % episode['description'])
 		self.PopupMenu(self.__issue_context_popup, pos)
+
 	#--------------------------------------------------------
 	def __handle_episode_context(self, pos=wx.DefaultPosition):
 #		self.__epi_context_popup.SetTitle(_('Episode %s') % self.__curr_node_data['description'])
 		self.PopupMenu(self.__epi_context_popup, pos)
+
 	#--------------------------------------------------------
 	def __handle_encounter_context(self, pos=wx.DefaultPosition):
 		self.PopupMenu(self.__enc_context_popup, pos)
+
 	#--------------------------------------------------------
 	# episode level
 	#--------------------------------------------------------
 	def __move_encounters(self, event):
 		episode = self.GetPyData(self.__curr_node)
 
-		gmNarrativeWidgets.move_progress_notes_to_another_encounter (
+		gmNarrativeWorkflows.move_progress_notes_to_another_encounter (
 			parent = self,
 			episodes = [episode['pk_episode']],
 			move_all = True
 		)
+
 	#--------------------------------------------------------
 	def __edit_episode(self, event):
 		gmEMRStructWidgets.edit_episode(parent = self, episode = self.__curr_node_data)
+
 	#--------------------------------------------------------
 	def __promote_episode_to_issue(self, evt):
 		gmEMRStructWidgets.promote_episode_to_issue(parent=self, episode = self.__curr_node_data, emr = self.__pat.emr)
+
 	#--------------------------------------------------------
 	def __delete_episode(self, event):
 		dlg = gmGuiHelpers.c2ButtonQuestionDlg (
@@ -603,6 +658,7 @@ class cEMRTree(wx.TreeCtrl, treemixin.ExpansionState):
 
 		if not gmEMRStructItems.delete_episode(episode = self.__curr_node_data):
 			gmDispatcher.send(signal = 'statustext', msg = _('Cannot delete episode. There is still clinical data recorded for it.'))
+
 	#--------------------------------------------------------
 	def __expand_episode_node(self, episode_node=None):
 		self.DeleteChildren(episode_node)
@@ -642,6 +698,7 @@ class cEMRTree(wx.TreeCtrl, treemixin.ExpansionState):
 			self.SetItemHasChildren(encounter_node, False)
 
 		self.SortChildren(episode_node)
+
 	#--------------------------------------------------------
 	# encounter level
 	#--------------------------------------------------------
@@ -650,7 +707,7 @@ class cEMRTree(wx.TreeCtrl, treemixin.ExpansionState):
 		node_parent = self.GetItemParent(self.__curr_node)
 		episode = self.GetPyData(node_parent)
 
-		gmNarrativeWidgets.move_progress_notes_to_another_encounter (
+		gmNarrativeWorkflows.move_progress_notes_to_another_encounter (
 			parent = self,
 			encounters = [encounter['pk_encounter']],
 			episodes = [episode['pk_episode']]
@@ -661,7 +718,7 @@ class cEMRTree(wx.TreeCtrl, treemixin.ExpansionState):
 		node_parent = self.GetItemParent(self.__curr_node)
 		episode = self.GetPyData(node_parent)
 
-		gmNarrativeWidgets.manage_progress_notes (
+		gmNarrativeWorkflows.manage_progress_notes (
 			parent = self,
 			encounters = [encounter['pk_encounter']],
 			episodes = [episode['pk_episode']]
@@ -694,6 +751,7 @@ class cEMRTree(wx.TreeCtrl, treemixin.ExpansionState):
 	#--------------------------------------------------------
 	def __edit_issue(self, event):
 		gmEMRStructWidgets.edit_health_issue(parent = self, issue = self.__curr_node_data)
+
 	#--------------------------------------------------------
 	def __delete_issue(self, event):
 		dlg = gmGuiHelpers.c2ButtonQuestionDlg (
@@ -717,10 +775,9 @@ class cEMRTree(wx.TreeCtrl, treemixin.ExpansionState):
 
 		dlg.Destroy()
 
-		try:
-			gmEMRStructItems.delete_health_issue(health_issue = self.__curr_node_data)
-		except gmExceptions.DatabaseObjectInUseError:
+		if not gmEMRStructItems.delete_health_issue(health_issue = self.__curr_node_data):
 			gmDispatcher.send(signal = 'statustext', msg = _('Cannot delete health issue. There is still clinical data recorded for it.'))
+
 	#--------------------------------------------------------
 	def __expand_issue_to_encounter_level(self, evt):
 
@@ -733,6 +790,7 @@ class cEMRTree(wx.TreeCtrl, treemixin.ExpansionState):
 		while epi.IsOk():
 			self.Expand(epi)
 			epi, epi_cookie = self.GetNextChild(self.__curr_node, epi_cookie)
+
 	#--------------------------------------------------------
 	def __expand_issue_node(self, issue_node=None):
 		self.DeleteChildren(issue_node)
@@ -752,6 +810,7 @@ class cEMRTree(wx.TreeCtrl, treemixin.ExpansionState):
 			self.SetItemHasChildren(episode_node, True)
 
 		self.SortChildren(issue_node)
+
 	#--------------------------------------------------------
 	def __expand_pseudo_issue_node(self, fake_issue_node=None):
 		self.DeleteChildren(fake_issue_node)
@@ -772,18 +831,26 @@ class cEMRTree(wx.TreeCtrl, treemixin.ExpansionState):
 			self.SetItemHasChildren(episode_node, True)
 
 		self.SortChildren(fake_issue_node)
+
 	#--------------------------------------------------------
 	# EMR level
+	#--------------------------------------------------------
+	def __print_emr(self, event):
+		gmFormWidgets.print_doc_from_template(parent = self)
+
 	#--------------------------------------------------------
 	def __create_issue(self, event):
 		gmEMRStructWidgets.edit_health_issue(parent = self, issue = None)
+
 	#--------------------------------------------------------
 	def __create_episode(self, event):
 		gmEMRStructWidgets.edit_episode(parent = self, episode = None)
+
 	#--------------------------------------------------------
 	def __create_soap_editor(self, event):
 		self.__cb__select_edit_mode(True)
 		self.__cb__add_soap_editor(problem = self.__curr_node_data, allow_same_problem = False)
+
 	#--------------------------------------------------------
 	def __document_allergy(self, event):
 		dlg = gmAllergyWidgets.cAllergyManagerDlg(parent=self, id=-1)
@@ -793,21 +860,27 @@ class cEMRTree(wx.TreeCtrl, treemixin.ExpansionState):
 			self.__populate_tree()
 		dlg.Destroy()
 		return
+
 	#--------------------------------------------------------
 	def __manage_procedures(self, event):
 		gmEMRStructWidgets.manage_performed_procedures(parent = self)
+
 	#--------------------------------------------------------
 	def __manage_family_history(self, event):
 		gmFamilyHistoryWidgets.manage_family_history(parent = self)
+
 	#--------------------------------------------------------
 	def __manage_hospital_stays(self, event):
 		gmEMRStructWidgets.manage_hospital_stays(parent = self)
+
 	#--------------------------------------------------------
 	def __manage_occupation(self, event):
 		gmDemographicsWidgets.edit_occupation()
+
 	#--------------------------------------------------------
 	def __manage_vaccinations(self, event):
 		gmVaccWidgets.manage_vaccinations(parent = self)
+
 	#--------------------------------------------------------
 	def __expand_to_issue_level(self, evt):
 
@@ -827,6 +900,7 @@ class cEMRTree(wx.TreeCtrl, treemixin.ExpansionState):
 				self.Collapse(epi)
 				epi, epi_cookie = self.GetNextChild(issue, epi_cookie)
 			issue, issue_cookie = self.GetNextChild(root_item, issue_cookie)
+
 	#--------------------------------------------------------
 	def __expand_to_episode_level(self, evt):
 
@@ -846,6 +920,7 @@ class cEMRTree(wx.TreeCtrl, treemixin.ExpansionState):
 				self.Collapse(epi)
 				epi, epi_cookie = self.GetNextChild(issue, epi_cookie)
 			issue, issue_cookie = self.GetNextChild(root_item, issue_cookie)
+
 	#--------------------------------------------------------
 	def __expand_to_encounter_level(self, evt):
 
@@ -865,9 +940,10 @@ class cEMRTree(wx.TreeCtrl, treemixin.ExpansionState):
 				self.Expand(epi)
 				epi, epi_cookie = self.GetNextChild(issue, epi_cookie)
 			issue, issue_cookie = self.GetNextChild(root_item, issue_cookie)
+
 	#--------------------------------------------------------
 	def __export_encounter_for_medistar(self, evt):
-		gmNarrativeWidgets.export_narrative_for_medistar_import (
+		gmNarrativeWorkflows.export_narrative_for_medistar_import (
 			parent = self,
 			soap_cats = u'soapu',
 			encounter = self.__curr_node_data
@@ -879,14 +955,18 @@ class cEMRTree(wx.TreeCtrl, treemixin.ExpansionState):
 
 		issues = [{
 			'description': _('Unattributed episodes'),
+			'laterality': None,
+			'diagnostic_certainty_classification': None,
 			'has_open_episode': False,
 			'pk_health_issue': None
 		}]
-
 		issues.extend(self.__pat.emr.health_issues)
-
 		for issue in issues:
-			issue_node =  self.AppendItem(root_node, issue['description'])
+			issue_node =  self.AppendItem(root_node, u'%s%s%s' % (
+				issue['description'],
+				gmTools.coalesce(issue['laterality'], u'', u' [%s]', none_equivalents = [None, u'na']),
+				gmTools.coalesce(issue['diagnostic_certainty_classification'], u'', u' [%s]')
+			))
 			self.SetItemBold(issue_node, issue['has_open_episode'])
 			self.SetItemPyData(issue_node, issue)
 			# fake it so we can expand it
@@ -894,11 +974,13 @@ class cEMRTree(wx.TreeCtrl, treemixin.ExpansionState):
 
 		self.SetItemHasChildren(root_node, (len(issues) != 0))
 		self.SortChildren(root_node)
+
 	#--------------------------------------------------------
 	# event handlers
 	#--------------------------------------------------------
 	def _on_narrative_mod_db(self, *args, **kwargs):
 		self.__update_text_for_selected_node()
+
 	#--------------------------------------------------------
 	def _on_episode_mod_db(self, *args, **kwargs):
 		self.__expanded_nodes = self.ExpansionState
@@ -1356,6 +1438,7 @@ class cEMRJournalPluginPnl(wxgEMRJournalPluginPnl.wxgEMRJournalPluginPnl):
 	def __init__(self, *args, **kwds):
 
 		wxgEMRJournalPluginPnl.wxgEMRJournalPluginPnl.__init__(self, *args, **kwds)
+		self._TCTRL_journal.disable_keyword_expansions()
 		self._TCTRL_journal.SetValue(u'')
 	#--------------------------------------------------------
 	# external API
@@ -1364,26 +1447,14 @@ class cEMRJournalPluginPnl(wxgEMRJournalPluginPnl.wxgEMRJournalPluginPnl):
 		self._TCTRL_journal.SetValue(u'')
 		exporter = gmPatientExporter.cEMRJournalExporter()
 		if self._RBTN_by_encounter.GetValue():
-			txt = StringIO.StringIO()
-			# FIXME: if journal is large this will error out, use generator/yield etc
-			# FIXME: turn into proper list
-			try:
-				exporter.export(txt)
-				self._TCTRL_journal.SetValue(txt.getvalue())
-			except ValueError:
-				_log.exception('cannot get EMR journal')
-				self._TCTRL_journal.SetValue (_(
-					'An error occurred while retrieving the EMR\n'
-					'in journal form for the active patient.\n\n'
-					'Please check the log file for details.'
-				))
-			txt.close()
+			fname = exporter.export_to_file_by_encounter(patient = gmPerson.gmCurrentPatient())
 		else:
-			fname = exporter.export_to_file_by_mod_time()
-			f = codecs.open(filename = fname, mode = 'rU', encoding = 'utf8', errors = 'replace')
-			for line in f:
-				self._TCTRL_journal.AppendText(line)
-			f.close()
+			fname = exporter.export_to_file_by_mod_time(patient = gmPerson.gmCurrentPatient())
+
+		f = io.open(fname, mode = 'rt', encoding = 'utf8', errors = 'replace')
+		for line in f:
+			self._TCTRL_journal.AppendText(line)
+		f.close()
 
 		self._TCTRL_journal.ShowPosition(self._TCTRL_journal.GetLastPosition())
 		return True
@@ -1394,25 +1465,278 @@ class cEMRJournalPluginPnl(wxgEMRJournalPluginPnl.wxgEMRJournalPluginPnl):
 		gmDispatcher.connect(signal = u'pre_patient_unselection', receiver = self._on_pre_patient_unselection)
 		gmDispatcher.connect(signal = u'post_patient_selection', receiver = self._on_post_patient_selection)
 		return True
+
 	#--------------------------------------------------------
-	# event handler
+	# event handlers
 	#--------------------------------------------------------
 	def _on_pre_patient_unselection(self):
 		self._TCTRL_journal.SetValue(u'')
 		return True
+
 	#--------------------------------------------------------
 	def _on_post_patient_selection(self):
 		if self.GetParent().GetCurrentPage() != self:
 			return True
 		self.repopulate_ui()
 		return True
+
 	#--------------------------------------------------------
 	def _on_order_by_encounter_selected(self, event):
 		self.repopulate_ui()
+
 	#--------------------------------------------------------
 	def _on_order_by_last_mod_selected(self, event):
 		self.repopulate_ui()
 
+	#--------------------------------------------------------
+	def _on_button_find_pressed(self, event):
+		self._TCTRL_journal.show_find_dialog(title = _('Find text in EMR Journal'))
+
+#================================================================
+from Gnumed.wxGladeWidgets import wxgEMRListJournalPluginPnl
+
+class cEMRListJournalPluginPnl(wxgEMRListJournalPluginPnl.wxgEMRListJournalPluginPnl):
+
+	def __init__(self, *args, **kwds):
+
+		wxgEMRListJournalPluginPnl.wxgEMRListJournalPluginPnl.__init__(self, *args, **kwds)
+
+		self._LCTRL_journal.select_callback = self._on_row_selected
+		self._TCTRL_details.SetValue(u'')
+
+		self.__load_timer = gmTimer.cTimer(callback = self._on_load_details, delay = 1000, cookie = 'EMRListJournalPluginDBLoadTimer')
+
+		self.__data = {}
+
+	#--------------------------------------------------------
+	# external API
+	#--------------------------------------------------------
+	def repopulate_ui(self):
+		self._LCTRL_journal.remove_items_safely()
+		self._TCTRL_details.SetValue(u'')
+
+		if self._RBTN_by_encounter.Value:		# (... is True:)
+			order_by = u'c_vej.encounter_started, c_vej.pk_episode, c_vej.src_table, scr, c_vej.modified_when'
+						#, c_vej.clin_when (should not make a relevant difference)
+			date_col_header = _('Encounter')
+			date_fields = ['encounter_started', 'modified_when']
+		elif self._RBTN_by_last_modified.Value:	# (... is True:)
+			order_by = u'c_vej.modified_when, c_vej.pk_episode, c_vej.src_table, scr'
+						#, c_vej.clin_when (should not make a relevant difference)
+			date_col_header = _('Modified')
+			date_fields = ['modified_when']
+		elif self._RBTN_by_item_time.Value:		# (... is True:)
+			order_by = u'c_vej.clin_when, c_vej.pk_episode, c_vej.src_table, scr, c_vej.modified_when'
+			date_col_header = _('Clinical time')
+			date_fields = ['clin_when', 'modified_when']
+		else:
+			raise ValueError('invalid EMR journal list sort state')
+
+		self._LCTRL_journal.set_columns([date_col_header, u'', _('Entry'), _('Who / When')])
+		self._LCTRL_journal.set_resize_column(3)
+
+		journal = gmPerson.gmCurrentPatient().emr.get_as_journal(order_by = order_by)
+
+		items = []
+		data = []
+		self.__data = {}
+		prev_date = None
+		for entry in journal:
+			if entry['narrative'].strip() == u'':
+				continue
+			soap_cat = gmSoapDefs.soap_cat2l10n[entry['soap_cat']]
+			who = u'%s (%s)' % (entry['modified_by'], entry['date_modified'])
+			try:
+				entry_date = gmDateTime.pydt_strftime(entry[date_fields[0]], u'%Y-%m-%d')
+			except KeyError:
+				entry_date = gmDateTime.pydt_strftime(entry[date_fields[1]], u'%Y-%m-%d')
+			if entry_date == prev_date:
+				date2show = u''
+			else:
+				date2show = entry_date
+				prev_date = entry_date
+			lines = entry['narrative'].strip().split(u'\n')
+			entry_line = lines[0].rstrip()
+			items.append([date2show, soap_cat, entry_line, who])
+			try:
+				self.__data[entry['src_table']]
+			except KeyError:
+				self.__data[entry['src_table']] = {}
+			try:
+				self.__data[entry['src_table']][entry['src_pk']]
+			except KeyError:
+				self.__data[entry['src_table']][entry['src_pk']] = {}
+				self.__data[entry['src_table']][entry['src_pk']]['entry'] = entry
+				self.__data[entry['src_table']][entry['src_pk']]['formatted_instance'] = None
+				if entry['encounter_started'] is None:
+					enc_duration = gmTools.u_diameter
+				else:
+					enc_duration = u'%s - %s' % (
+						gmDateTime.pydt_strftime(entry['encounter_started'], '%Y %b %d  %H:%M'),
+						gmDateTime.pydt_strftime(entry['encounter_last_affirmed'], '%H:%M')
+					)
+				self.__data[entry['src_table']][entry['src_pk']]['formatted_header'] = _(
+					u'Chart entry: %s       [#%s in %s]\n'
+					u' Modified: %s by %s (%s rev %s)\n'
+					u'\n'
+					u'Health issue: %s%s\n'
+					u'Episode: %s%s\n'
+					u'Encounter: %s%s'
+				) % (
+					gmClinicalRecord.format_clin_root_item_type(entry['src_table']),
+					entry['src_pk'],
+					entry['src_table'],
+					entry['date_modified'],
+					entry['modified_by'],
+					gmTools.u_arrow2right,
+					entry['row_version'],
+					gmTools.coalesce(entry['health_issue'], gmTools.u_diameter, u'%s'),
+					gmTools.bool2subst(entry['issue_active'], u' (' + _('active') + u')', u' (' + _('inactive') + u')', u''),
+					gmTools.coalesce(entry['episode'], gmTools.u_diameter, u'%s'),
+					gmTools.bool2subst(entry['episode_open'], u' (' +  _('open') + u')', u' (' +  _('closed') + u')', u''),
+					enc_duration,
+					gmTools.coalesce(entry['encounter_l10n_type'], u'', u' (%s)'),
+				)
+				self.__data[entry['src_table']][entry['src_pk']]['formatted_root_item'] = _(
+					u'%s\n'
+					u'\n'
+					u'                        rev %s (%s) by %s in <%s>'
+				) % (
+					entry['narrative'].strip(),
+					entry['row_version'],
+					entry['date_modified'],
+					entry['modified_by'],
+					entry['src_table']
+				)
+			data.append ({
+				'table': entry['src_table'],
+				'pk': entry['src_pk']
+			})
+			if len(lines) > 1:
+				lines = lines[1:]
+				idx = 0
+				last_line = len(lines)
+				for entry_line in lines:
+					idx += 1
+					if entry_line.strip() == u'':
+						continue
+					if idx == last_line:
+						bar = gmTools.u_box_bottom_left_arc
+					else:
+						bar = gmTools.u_box_vert_light_4dashes
+					items.append([u'', bar, entry_line.rstrip(), who])
+					data.append ({
+						'table': entry['src_table'],
+						'pk': entry['src_pk']
+					})
+
+		self._LCTRL_journal.set_string_items(items)
+		self._LCTRL_journal.set_column_widths([wx.LIST_AUTOSIZE, wx.LIST_AUTOSIZE, wx.LIST_AUTOSIZE, wx.LIST_AUTOSIZE_USEHEADER])
+		self._LCTRL_journal.set_data(data)
+
+		self._LCTRL_journal.SetFocus()
+		return True
+
+	#--------------------------------------------------------
+	# internal helpers
+	#--------------------------------------------------------
+	def __register_events(self):
+		gmDispatcher.connect(signal = u'pre_patient_unselection', receiver = self._on_pre_patient_unselection)
+		gmDispatcher.connect(signal = u'post_patient_selection', receiver = self._on_post_patient_selection)
+		return True
+
+	#--------------------------------------------------------
+	# event handlers
+	#--------------------------------------------------------
+	def _on_pre_patient_unselection(self):
+		self._LCTRL_journal.remove_items_safely()
+		self._TCTRL_details.SetValue(u'')
+		self.__data = {}
+		return True
+
+	#--------------------------------------------------------
+	def _on_post_patient_selection(self):
+		if self.GetParent().GetCurrentPage() != self:
+			return True
+		self.repopulate_ui()
+		return True
+
+	#--------------------------------------------------------
+	def _on_row_selected(self, evt):
+		# FIXME: work on all selected
+		data = self._LCTRL_journal.get_item_data(item_idx = evt.Index)
+		if self.__data[data['table']][data['pk']]['formatted_instance'] is None:
+			txt = _(
+				u'%s\n'
+				u'%s\n'
+				u'%s'
+			) % (
+				self.__data[data['table']][data['pk']]['formatted_header'],
+				gmTools.u_box_horiz_4dashes * 40,
+				self.__data[data['table']][data['pk']]['formatted_root_item']
+			)
+
+			self._TCTRL_details.SetValue(txt)
+			self.__load_timer.Stop()
+			self.__load_timer.Start(oneShot = True)
+			return
+
+		txt = _(
+			u'%s\n'
+			u'%s\n'
+			u'%s'
+		) % (
+			self.__data[data['table']][data['pk']]['formatted_header'],
+			gmTools.u_box_horiz_4dashes * 40,
+			self.__data[data['table']][data['pk']]['formatted_instance']
+		)
+		self._TCTRL_details.SetValue(txt)
+
+	#--------------------------------------------------------
+	def _on_load_details(self, cookie):
+		data = self._LCTRL_journal.get_selected_item_data(only_one = True)
+		if self.__data[data['table']][data['pk']]['formatted_instance'] is None:
+			self.__data[data['table']][data['pk']]['formatted_instance'] = gmClinicalRecord.format_clin_root_item(data['table'], data['pk'], patient = gmPerson.gmCurrentPatient())
+		txt = _(
+			u'%s\n'
+			u'%s\n'
+			u'%s'
+		) % (
+			self.__data[data['table']][data['pk']]['formatted_header'],
+			gmTools.u_box_horiz_4dashes * 40,
+			self.__data[data['table']][data['pk']]['formatted_instance']
+		)
+		wx.CallAfter(self._TCTRL_details.SetValue, txt)
+
+	#--------------------------------------------------------
+	def _on_order_by_encounter_selected(self, event):
+		self.repopulate_ui()
+
+	#--------------------------------------------------------
+	def _on_order_by_last_mod_selected(self, event):
+		self.repopulate_ui()
+
+	#--------------------------------------------------------
+	def _on_order_by_item_time_selected(self, event):
+		self.repopulate_ui()
+
+	#--------------------------------------------------------
+	def _on_order_by_item_time_selected(self, event):
+		event.Skip()
+		self.repopulate_ui()
+
+	#--------------------------------------------------------
+	def _on_edit_button_pressed(self, event):
+		event.Skip()
+
+	#--------------------------------------------------------
+	def _on_delete_button_pressed(self, event):
+		event.Skip()
+
+	#--------------------------------------------------------
+#	def _on_button_find_pressed(self, event):
+#		self._TCTRL_details.show_find_dialog(title = _('Find text in EMR Journal'))
+
 #================================================================
 # MAIN
 #----------------------------------------------------------------
@@ -1442,7 +1766,7 @@ if __name__ == '__main__':
 				patient.cleanup()
 			except:
 				print "error cleaning up patient"
-	except StandardError:
+	except Exception:
 		_log.exception("unhandled exception caught !")
 		# but re-raise them
 		raise
