@@ -96,7 +96,7 @@ if __name__ == "__main__":
 				patient.cleanup()
 			except:
 				print "error cleaning up patient"
-	except StandardError:
+	except Exception:
 		_log.exception("unhandled exception caught !")
 		# but re-raise them
 		raise
